﻿//-----------------------------------------------------------------------
// <copyright file="DLLCaller.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Net;
using System.Runtime.InteropServices;

using CenturySystems.FLNet;

namespace CenturySystems.FLPCI
{
    /// <summary>
    /// DLLの関数 FL_Initialize() を呼ぶ際に使用する構造体
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    internal struct _InitInfo
    {
        /// <summary>
        /// IPアドレス
        /// </summary>
        private readonly uint ip_adr;

        /// <summary>
        /// デバイス番号
        /// </summary>
        private readonly ushort dev_no;

        /// <summary>
        /// コモン領域1先頭アドレス
        /// </summary>
        private readonly ushort cmn1_adr;

        /// <summary>
        /// コモン領域1サイズ
        /// </summary>
        private readonly ushort cmn1_size;

        /// <summary>
        /// コモン領域2先頭アドレス
        /// </summary>
        private readonly ushort cmn2_adr;

        /// <summary>
        /// コモン領域2サイズ
        /// </summary>
        private readonly ushort cmn2_size;

        /// <summary>
        /// トークン監視タイムアウト値
        /// </summary>
        private readonly byte tw;

        /// <summary>
        /// 最小フレーム間隔
        /// </summary>
        private readonly byte mft;

        /// <summary>
        /// ベンダ名
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 11)]
        private readonly string vender;

        /// <summary>
        /// メーカー名
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 11)]
        private readonly string maker;

        /// <summary>
        /// ノード名
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 11)]
        private readonly string name;

        /// <summary>
        /// _InitInfoクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="ipAddress">IPアドレス</param>
        /// <param name="nodeInitInfo">ノード初期化情報</param>
        /// <param name="deviceNumber">デバイス番号</param>
        /// <exception cref="System.ArgumentNullException">引数がnull</exception>
        /// <exception cref="System.ArgumentOutOfRangeException">引数の範囲が異常</exception>
        public _InitInfo(IPAddress ipAddress, NodeInitInfo nodeInitInfo, ushort deviceNumber)
            : this()
        {
            if (ipAddress == null)
            {
                throw new ArgumentNullException("ipAddress");
            }

            if (nodeInitInfo == null)
            {
                throw new ArgumentNullException("nodeInitInfo");
            }

            if (deviceNumber > FLPCIClass.DeviceMax)
            {
                throw new ArgumentOutOfRangeException("deviceNumber");
            }

            //// ipAddress, nodeInitInfoの範囲チェックは行わない(呼び出し元でされているはず)

            byte[] ipBytes = ipAddress.GetAddressBytes();
            this.ip_adr = ((uint)ipBytes[0] << 24) + ((uint)ipBytes[1] << 16) + ((uint)ipBytes[2] << 8) + (uint)ipBytes[3];
            this.dev_no = deviceNumber;
            this.cmn1_adr = nodeInitInfo.Common1Address;
            this.cmn1_size = nodeInitInfo.Common1Size;
            this.cmn2_adr = nodeInitInfo.Common2Address;
            this.cmn2_size = nodeInitInfo.Common2Size;
            this.tw = nodeInitInfo.TokenWatchTimeout;
            this.mft = nodeInitInfo.MinFrameintervalTime;
            this.vender = "CENTURYSYS";
            this.maker = "S-0616    ";
            this.name = nodeInitInfo.NodeName;
        }
    }

    /// <summary>
    /// DLLの関数 ReadMessage(), WriteMessage() を呼ぶ際に使用する構造体
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    internal struct _MessageInfo
    {
        /// <summary>
        /// ノード番号
        /// </summary>
        internal readonly byte node;

        /// <summary>
        /// リザルトコード
        /// </summary>
        internal readonly byte result;

        /// <summary>
        /// トランザクションコード
        /// </summary>
        internal readonly uint tcd;

        /// <summary>
        /// バイトサイズ
        /// </summary>
        internal readonly ushort bc;

        /// <summary>
        /// 仮想メモリ空間アドレス
        /// </summary>
        internal readonly uint addr;

        /// <summary>
        /// 仮想メモリ空間サイズ
        /// </summary>
        internal readonly ushort length;

        /// <summary>
        /// _MessageInfoクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="messageInfo">メッセージ情報</param>
        /// <exception cref="System.ArgumentNullException">引数がnull</exception>
        public _MessageInfo(MessageInfo messageInfo)
        {
            if (messageInfo == null)
            {
                throw new ArgumentNullException("messageInfo");
            }

            this.node = messageInfo.NodeNumber;
            this.result = (byte)messageInfo.ResultCode;
            this.tcd = messageInfo.TCD;
            this.bc = (ushort)messageInfo.BSize;
            this.addr = messageInfo.MAddr;
            this.length = (ushort)messageInfo.MSize;
        }

        /// <summary>
        /// 自分の情報から新しいMessageInfoクラスのインスタンスを作成して返す
        /// </summary>
        /// <param name="origData">メッセージのデータ部</param>
        /// <returns>新しいMessageInfoクラスのインスタンス</returns>
        /// <exception cref="System.ArgumentNullException">引数がnull</exception>
        /// <exception cref="System.ArgumentOutOfRangeException">引数が異常</exception>
        internal MessageInfo MakeMessageInfo(byte[] origData)
        {
            if (origData == null)
            {
                throw new ArgumentNullException("origData");
            }

            var nodeNumber = this.node;
            if (!ResultCode.IsDefined(typeof(ResultCode), this.result))
            {
                throw new ArgumentOutOfRangeException("resultcode");
            }

            var resultCode = (ResultCode)Enum.ToObject(typeof(ResultCode), this.result);
            var tcd = (ushort)this.tcd;
            var bSize = this.bc;
            var mAddr = this.addr;
            var mSize = this.length;

            // 仮想アドレスを使う場合と使わない場合とでdataの大きさの元が異なる
            var length = Math.Max(bSize, mSize);
            var data = new byte[length];
            var copyLength = Math.Min(length, origData.Length);
            Array.Copy(origData, data, copyLength);

            var returnValue = new MessageInfo(nodeNumber, resultCode, tcd, bSize, mAddr, mSize, data);
            return returnValue;
        }
    }

    /// <summary>
    /// DLLの関数 GetNodeInfo()を呼ぶ際に必要な構造体
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    internal struct _NodeInfo
    {
        /// <summary>
        /// 参加/離脱
        /// </summary>
        internal readonly sbyte sanka;

        /// <summary>
        /// 上位層の状態
        /// </summary>
        internal readonly ushort uls;

        /// <summary>
        /// コモン領域1アドレス
        /// </summary>
        internal readonly ushort cmn1_addr;

        /// <summary>
        /// コモン領域1サイズ
        /// </summary>
        internal readonly ushort cmn1_size;

        /// <summary>
        /// コモン領域2アドレス
        /// </summary>
        internal readonly ushort cmn2_addr;

        /// <summary>
        /// コモン領域2サイズ
        /// </summary>
        internal readonly ushort cmn2_size;

        /// <summary>
        /// リフレッシュサイクル許容時間
        /// </summary>
        internal readonly ushort rct;

        /// <summary>
        /// トークン監視時間
        /// </summary>
        internal readonly byte tw;

        /// <summary>
        /// 最小フレーム間隔
        /// </summary>
        internal readonly byte mft;

        /// <summary>
        /// リンクの状態
        /// </summary>
        internal readonly byte link_status;

        /// <summary>
        /// 自ノードの状態
        /// </summary>
        internal readonly byte my_status;

        /// <summary>
        /// ベンダ名
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 11)]
        internal readonly string vender;

        /// <summary>
        /// メーカー名
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 11)]
        internal readonly string maker;

        /// <summary>
        /// ノード名
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 11)]
        internal readonly string node;
    }

    /// <summary>
    /// DLL関数 GetRingInfo() の呼び出しに必要な構造体
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    internal struct _SankaReqTable
    {
        /// <summary>
        /// ノード数
        /// </summary>
        internal readonly byte node_cnt;

        /// <summary>
        /// 参加ノードNo.
        /// </summary>
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 254)]
        internal readonly byte[] node_no;
    }

    /// <summary>
    /// DLL関数 GetNetInfo() の呼び出しに必要な構造体
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    internal struct _NetInfo
    {
        /// <summary>
        /// リフレッシュサイクル設定値
        /// </summary>
        internal readonly ushort rct;

        /// <summary>
        /// リフレッシュサイクル測定値(現在)
        /// </summary>
        internal readonly ushort rcm;
        
        /// <summary>
        /// リフレッシュサイクル測定値(最大)
        /// </summary>
        internal readonly ushort rcm_max;

        /// <summary>
        /// リフレッシュサイクル測定値(最小)
        /// </summary>
        internal readonly ushort rcm_min;

        /// <summary>
        /// リンクの状態
        /// </summary>
        internal readonly byte link_status;
    }

    /// <summary>
    /// DLL関数 GetErrInfo() の呼び出しに必要な構造体
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    internal struct _ErrInfo
    {
        /// <summary>
        /// エラーコード
        /// </summary>
        internal readonly ushort err_code;

        /// <summary>
        /// メッセージ長
        /// </summary>
        internal readonly ushort length;

        /// <summary>
        /// メッセージ
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
        internal readonly string err_msg;
    }

    /// <summary>
    /// DLL関数 ReadUls_All()の呼び出しに必要な構造体
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    internal struct _UlsInfo
    {
        /// <summary>
        /// 上位層の状態
        /// </summary>
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 254)]
        internal readonly ushort[] uls;

        /// <summary>
        /// リンクの状態
        /// </summary>
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 254)]
        internal readonly byte[] lks;
    }

    /// <summary>
    /// FL-PCIクラス
    /// </summary>
    public partial class FLPCIClass
    {
        /// <summary>
        /// デバイス番号の最大値
        /// </summary>
        internal const ushort DeviceMax = 3;

        /// <summary>
        /// 上位層の状態を表すushort型の数値から、UPLStatusクラスの新しいインスタンスを初期化します。
        /// DLLの呼び出しの際に使用します。
        /// </summary>
        /// <param name="uls">上位層の状態を表す数値(DLL仕様書の3.15参照)</param>
        /// <returns>UPLStatusクラスのインスタンス</returns>
        /// <exception cref="System.ArgumentOutOfRangeException">エラーNo.が範囲外</exception>
        internal static UPLStatus CreateUPLStatus(ushort uls)
        {
            bool run = (uls & 0x8000) != 0;
            bool alarm = (uls & 0x4000) != 0;
            bool warning = (uls & 0x2000) != 0;
            ushort errorNumber = (ushort)(uls & 0x0fff);
            return new UPLStatus(run, alarm, warning, errorNumber);
        }

        /// <summary>
        /// UPLStatusをushort型の数値に変換します。DLLの呼び出しの際に使用します。
        /// </summary>
        /// <param name="upl">UPLStatus</param>
        /// <returns>上位層の状態を表す数値(DLL仕様書の3.15参照)</returns>
        private static ushort UPLStatusToUShort(UPLStatus upl)
        {
            if (upl == null)
            {
                throw new ArgumentNullException("upl");
            }

            ushort returnValue = 0;
            if (upl.Run)
            {
                returnValue += 0x8000;
            }

            if (upl.Alarm)
            {
                returnValue += 0x4000;
            }

            if (upl.Warning)
            {
                returnValue += 0x2000;
            }

            returnValue += upl.ErrorNumber;
            return returnValue;
        }

        /// <summary>
        /// DLL呼び出しの結果生じたエラーを例外にして投げる
        /// </summary>
        /// <param name="rc">DLL呼び出しの返り値</param>
        /// <param name="functionName">呼び出したDLLの関数名</param>
        /// <exception cref="CenturySystems.FLPCI.InternalException">rc=0(正常)の時(これはAPI内部のバグ)</exception>
        /// <exception cref="System.ArgumentException">パラメータエラー(rc=2)</exception>
        /// <exception cref="System.TimeoutException">タイムアウト(rc=8)</exception>
        /// <exception cref="CenturySystems.FLPCI.HardwareException">ハードウェア障害の時(rc=9)</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">その他のエラー(これはAPI内部かDLLのバグ)</exception>
        private void ThrowDLLException(
            ushort rc,
            string functionName)
        {
            switch (rc)
            {
                case 0:
                    throw new InternalException("rc=0 in ThrowDLLException");
                case 2:
                    throw new ArgumentException();
                case 4:
                    throw new DLLException(functionName, "No data");
                case 8:
                    throw new TimeoutException();
                case 9:
                    var errInf = this.GetErrInfo();
                    throw new HardwareException(errInf.Code, string.Format("{0}", errInf.Message));
                case 10:
                    throw new DLLException(functionName, "Device open error (file create)");
                case 11:
                    throw new DLLException(functionName, "Device open error (device get)");
                case 12:
                    throw new DLLException(functionName, "Device open error (pointer get)");
                case 14:
                    throw new InternalException("Not open: deviceNumber = " + this.DeviceNumber);
                case 19:
                    throw new DLLException(functionName, "mutex get error");
                default:
                    throw new DLLException(functionName, "Unknown (rc=" + rc + ")");
            }
        }

        /// <summary>
        /// ネイティブメソッドの呼び出し用クラス
        /// </summary>
        internal static class NativeMethods
        {
            [DllImport("flpci_mul.dll")]
            internal static extern ushort FL_Initialize(
                [In] ref _InitInfo init_prm);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort DeviceOpen(
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort WriteCyclicALL(
                [In] ushort[] buf1,
                [In] ushort[] buf2,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort WriteCyclicRelative(
                ushort common,
                ushort offset,
                ushort size,
                [In] ushort[] buff,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort ReadCyclicAbsolute(
                ushort common,
                ushort offset,
                ushort size,
                [Out] ushort[] buff,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort ReadCyclicRelative(
                ushort node,
                ushort common,
                ushort offset,
                ushort size,
                [Out] ushort[] buff,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort ReadCyclicALL(
                ushort node,
                [Out] ushort[] buf1,
                [Out] ushort[] buf2,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort WriteMessage(
                [In] ref _MessageInfo msnd_prm,
                [In] byte[] buff,
                ushort wait_flg,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort CheckMessageEnd(
                out ushort node,
                out ushort tcd,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort ReadMessage(
                out _MessageInfo mrcv_prm,
                [Out] byte[] buff,
                ushort wait_flg,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort GetNodeInfo(
                ushort node,
                out _NodeInfo node_inf,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort GetRingInfo(
                out _SankaReqTable ring_inf,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort GetNetInfo(
                out _NetInfo net_inf,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort GetLogInfo(
                [Out] uint[] log_buff,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort SetUplStatus(
                ushort uls,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort GetErrInfo(
                out _ErrInfo err_inf,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort DeviceClose(
                ushort dev_no,
                ushort reset_f);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort ReadUls_All(
                out _UlsInfo UlsInfo,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort Dll_Version();

            [DllImport("flpci_mul.dll")]
            internal static extern ushort Firmware_Version(
                out ushort version,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort Set_Speed(
                uint speed,
                ushort dev_no);

            [DllImport("flpci_mul.dll")]
            internal static extern ushort Get_Speed(
                out uint speed,
                out uint duplex,
                ushort dev_no);
        }
    }
}