﻿//-----------------------------------------------------------------------
// <copyright file="FLNet.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;

namespace CenturySystems.FLNet
{
    /// <summary>
    /// TCDコード
    /// </summary>
    public enum TCDCode : ushort
    {
        /// <summary>
        /// 透過型メッセージ59999
        /// </summary>
        TransparentMessage59999 = 59999,

        /// <summary>
        /// バイトブロックリード要求
        /// </summary>
        ByteBlockReadReq = 65003,

        /// <summary>
        /// バイトブロックライト要求
        /// </summary>
        ByteBlockWriteReq = 65004,

        /// <summary>
        /// ワードブロックリード要求
        /// </summary>
        WordBlockReadReq = 65005,

        /// <summary>
        /// ワードブロックライト要求
        /// </summary>
        WordBlockWriteReq = 65006,

        /// <summary>
        /// ネットワークパラメータリード要求
        /// </summary>
        NetworkParameterReadReq = 65007,

        /// <summary>
        /// ネットワークパラメータライト要求
        /// </summary>
        NetworkParameterWriteReq = 65008,

        /// <summary>
        /// 停止司令要求
        /// </summary>
        StopReq = 65009,

        /// <summary>
        /// 運転司令要求
        /// </summary>
        RunReq = 65010,

        /// <summary>
        /// プロファイルリード要求
        /// </summary>
        ProfileReadReq = 65011,

        /// <summary>
        /// ログデータリード要求
        /// </summary>
        LogDataReadReq = 65013,

        /// <summary>
        /// ログデータクリア要求
        /// </summary>
        LogDataClearReq = 65014,

        /// <summary>
        /// メッセージ折り返し要求
        /// </summary>
        MessageLoopbackReq = 65015,

        /// <summary>
        /// ベンダ固有メッセージ要求
        /// </summary>
        VenderMessageReq = 65016,

        /// <summary>
        /// バイトブロックリード応答
        /// </summary>
        ByteBlockReadRsp = 65203,

        /// <summary>
        /// バイトブロックライト応答
        /// </summary>
        ByteBlockWriteRsp = 65204,

        /// <summary>
        /// ワードブロックリード応答
        /// </summary>
        WordBlockReadRsp = 65205,

        /// <summary>
        /// ワードブロックライト応答
        /// </summary>
        WordBlockWriteRsp = 65206,

        /// <summary>
        /// ネットワークパラメータリード応答
        /// </summary>
        NetworkParameterReadRsp = 65207,

        /// <summary>
        /// ネットワークパラメータライト応答
        /// </summary>
        NetworkParameterWriteRsp = 65208,

        /// <summary>
        /// 停止指令応答
        /// </summary>
        StopRsp = 65209,

        /// <summary>
        /// 運転指令応答
        /// </summary>
        RunRsp = 65210,

        /// <summary>
        /// プロファイルリード応答
        /// </summary>
        ProfileReadRsp = 65211,

        /// <summary>
        /// ログデータリード応答
        /// </summary>
        LogDataReadRsp = 65213,

        /// <summary>
        /// ログデータクリア応答
        /// </summary>
        LogDataClearRsp = 65214,

        /// <summary>
        /// メッセージ折り返し応答
        /// </summary>
        MessageLoopbackRsp = 65215,

        /// <summary>
        /// ベンダ固有メッセージ応答
        /// </summary>
        VenderMessageRsp = 65216,
    }

    /// <summary>
    /// 応答メッセージの種別 (M_RLT)
    /// </summary>
    public enum ResultCode : byte
    {
        /// <summary>
        /// 正常
        /// </summary>
        Normal = 0,

        /// <summary>
        /// 異常
        /// </summary>
        Abnormal = 1,

        /// <summary>
        /// 非実装
        /// </summary>
        NotImplement = 2,
    }

    /// <summary>
    /// FL-net用ユーティリティ
    /// </summary>
    /// <remarks>
    /// 定数、範囲チェックルーチンなど
    /// </remarks>
    public static class FLNetUtil
    {
        /// <summary>
        /// ノード番号の最大値
        /// </summary>
        public const byte NodeNumberMax = 254;

        /// <summary>
        /// 1:N (ブロードキャスト) 通信の際のノード番号
        /// </summary>
        public const byte NodeNumberBroadcast = 255;

        /// <summary>
        /// コモン領域それぞれの大きさの最大値。
        /// </summary>
        public static readonly ushort[] CommonSizeMax = { 0, 512, 8192 };

        /// <summary>
        /// コモン領域それぞれのアドレスの最大値
        /// </summary>
        private static readonly ushort[] CommonAddressMax = { 0, (ushort)(CommonSizeMax[1] - 1), (ushort)(CommonSizeMax[2] - 1) };

        /// <summary>
        /// ノードNo.が仕様の範囲(1-254)に入っているかのチェック。
        /// </summary>
        /// <param name="nodeNumber">ノードNo.</param>
        /// <param name="permitBroadcast">trueならブロードキャストアドレス(255)も許容する</param>
        /// <exception cref="System.ArgumentOutOfRangeException">ノードNo.が範囲外の場合</exception>
        public static void CheckNodeNumber(uint nodeNumber, bool permitBroadcast)
        {
            if (nodeNumber == 0)
            {
                throw new ArgumentOutOfRangeException("Node Number");
            }

            if (!permitBroadcast)
            {
                if (nodeNumber > NodeNumberMax)
                {
                    throw new ArgumentOutOfRangeException("Node Number");
                }
            }
        }

        /// <summary>
        /// コモン番号が仕様の範囲(1-2)に入っているかのチェック
        /// </summary>
        /// <param name="commonNumber">コモン番号</param>
        /// <exception cref="System.ArgumentOutOfRangeException">コモン番号が範囲外の場合</exception>
        /// <remarks>コモン番号は1オリジン</remarks>
        public static void CheckCommonNumber(uint commonNumber)
        {
            if ((commonNumber == 0) || (commonNumber > 2))
            {
                throw new ArgumentOutOfRangeException("Common Number");
            }
        }

        /// <summary>
        /// コモン領域を読み書きするアドレス・オフセットが仕様の範囲に入っているかのチェック
        /// </summary>
        /// <param name="commonNumber">コモン番号</param>
        /// <param name="address">アドレス</param>
        /// <exception cref="System.ArgumentOutOfRangeException">コモン番号かアドレスが範囲外の場合</exception>
        public static void CheckAddress(uint commonNumber, uint address)
        {
            CheckCommonNumber(commonNumber);
            if (address > CommonAddressMax[commonNumber])
            {
                throw new ArgumentOutOfRangeException("address");
            }
        }

        /// <summary>
        /// コモン領域を読み書きするサイズが仕様の範囲に入っているかのチェック
        /// </summary>
        /// <param name="commonNumber">コモン番号</param>
        /// <param name="size">サイズ</param>
        /// <exception cref="System.ArgumentOutOfRangeException">コモン領域かサイズが範囲外の場合</exception>
        public static void CheckSize(uint commonNumber, uint size)
        {
            CheckCommonNumber(commonNumber);
            if (size > CommonSizeMax[commonNumber])
            {
                throw new ArgumentOutOfRangeException("size");
            }
        }
    }

    /// <summary>
    /// 上位層の状態(ULS)
    /// </summary>
    public class UPLStatus
    {
        /// <summary>
        /// エラーNo.の最大値
        /// </summary>
        public const ushort ErrorNumberMax = 0xfff;

        /// <summary>
        /// エラー番号
        /// </summary>
        private ushort errorNumber;

        /// <summary>
        /// UPLStatusクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="run">Run/Stop状態</param>
        /// <param name="alarm">Alarm状態</param>
        /// <param name="warning">Warning状態</param>
        /// <param name="errorNumber">エラーNo. (0～UPLStatus.ErrorNumberMax)</param>
        /// <exception cref="System.ArgumentOutOfRangeException">エラーNo.が範囲外</exception>
        public UPLStatus(bool run, bool alarm, bool warning, ushort errorNumber)
        {
            this.Run = run;
            this.Alarm = alarm;
            this.Warning = warning;
            this.ErrorNumber = errorNumber;
        }

        /// <summary>
        /// Run/Stop状態
        /// </summary>
        public bool Run { get; set; }

        /// <summary>
        /// Alarm状態
        /// </summary>
        public bool Alarm { get; set; }

        /// <summary>
        /// Warning状態
        /// </summary>
        public bool Warning { get; set; }

        /// <summary>
        /// エラーNo. (0～UPLStatus.ErrorNumberMax)
        /// </summary>
        public ushort ErrorNumber
        {
            get
            {
                return this.errorNumber;
            }

            set
            {
                if (value > ErrorNumberMax)
                {
                    throw new ArgumentOutOfRangeException("ErrorNumber");
                }

                this.errorNumber = value;
            }
        }
    }

    /// <summary>
    /// リンク状態(LKS)
    /// </summary>
    public class LinkStatus
    {
        /// <summary>
        /// LinkStatusクラスの新しいインスタンスをデフォルト値(全てfalse)で初期化します。
        /// </summary>
        public LinkStatus()
        {
            this.UplError = false;
            this.EnableCommonMemoryData = false;
            this.FinishCommonMemoryConfig = false;
            this.DuplicateAddress = false;
        }

        /// <summary>
        /// LinkStatusクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="uplError">上位層動作信号エラー</param>
        /// <param name="enableCommonMemoryData">コモンメモリ・データ有効通知</param>
        /// <param name="finishCommonMemoryConfig">コモンメモリ(アドレス・サイズ)設定完了</param>
        /// <param name="duplicateAddress">アドレス重複検知</param>
        public LinkStatus(bool uplError, bool enableCommonMemoryData, bool finishCommonMemoryConfig, bool duplicateAddress)
        {
            this.UplError = uplError;
            this.EnableCommonMemoryData = enableCommonMemoryData;
            this.FinishCommonMemoryConfig = finishCommonMemoryConfig;
            this.DuplicateAddress = duplicateAddress;
        }

        /// <summary>
        /// 上位層動作信号エラー
        /// </summary>
        public bool UplError { get; set; }

        /// <summary>
        /// コモンメモリ・データ有効通知
        /// </summary>
        public bool EnableCommonMemoryData { get; set; }

        /// <summary>
        /// コモンメモリ(アドレス・サイズ)設定完了
        /// </summary>
        public bool FinishCommonMemoryConfig { get; set; }

        /// <summary>
        /// アドレス重複検知
        /// </summary>
        public bool DuplicateAddress { get; set; }
    }

    /// <summary>
    /// メッセージを表すクラス
    /// </summary>
    public class MessageInfo
    {
        /// <summary>
        /// ノードNo. (1-254および255(ブロードキャスト))
        /// </summary>
        private byte nodeNumber;

        /// <summary>
        /// バイトサイズ (0-1024)
        /// </summary>
        private ushort bSize;

        /// <summary>
        /// 仮想アドレスデータサイズ (0-1024)
        /// </summary>
        private ushort mSize;

        /// <summary>
        /// 送受信データ(最大1024バイト)
        /// </summary>
        private byte[] data;

        /// <summary>
        /// MessageInfoクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="nodeNumber">送信先ノードNo.(1-254or255)</param>
        /// <param name="resultCode">応答メッセージの種別</param>
        /// <param name="tcd">トランザクションコード</param>
        /// <param name="bSize">バイトサイズ</param>
        /// <param name="mAddr">仮想アドレス空間アドレス</param>
        /// <param name="mSize">仮想アドレス空間サイズ</param>
        /// <param name="data">送信データ</param>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        /// <exception cref="System.ArgumentNullException">dataがnull</exception>
        public MessageInfo(
            byte nodeNumber,
            ResultCode resultCode,
            ushort tcd,
            ushort bSize,
            uint mAddr,
            ushort mSize,
            byte[] data)
        {
            this.NodeNumber = nodeNumber;
            this.ResultCode = resultCode;
            this.TCD = tcd;
            this.BSize = bSize;
            this.MAddr = mAddr;
            this.MSize = mSize;
            this.Data = data;
        }

        /// <summary>
        /// ノードNo. (1-254および255(ブロードキャスト))
        /// </summary>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        public byte NodeNumber
        {
            get
            {
                return this.nodeNumber;
            }

            set
            {
                FLNetUtil.CheckNodeNumber(value, true);
                this.nodeNumber = value;
            }
        }

        /// <summary>
        /// 応答メッセージの種別
        /// </summary>
        public ResultCode ResultCode { get; set; } // 範囲チェックはない

        /// <summary>
        /// トランザクションコード
        /// </summary>
        public ushort TCD { get; set; }　// 範囲チェックはない

        /// <summary>
        /// バイトサイズ (0-1024)
        /// </summary>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        public ushort BSize
        {
            get
            {
                return this.bSize;
            }

            set
            {
                if (value > 1024)
                {
                    throw new ArgumentOutOfRangeException("BC");
                }

                this.bSize = value;
            }
        }

        /// <summary>
        /// 仮想アドレス空間アドレス
        /// </summary>
        public uint MAddr { get; set; } // 範囲チェックはない

        /// <summary>
        /// バイトサイズ (0-1024)
        /// </summary>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        public ushort MSize
        {
            get
            {
                return this.mSize;
            }

            set
            {
                if (value > 1024)
                {
                    throw new ArgumentOutOfRangeException("MSize");
                }

                this.mSize = value;
            }
        }

        /// <summary>
        /// 送受信データ(最大1024バイト)
        /// </summary>
        /// <exception cref="System.ArgumentNullException">データがnull</exception>
        /// <exception cref="System.ArgumentOutOfRangeException">データの大きさが仕様の範囲外</exception>
        public byte[] Data
        {
            get
            {
                return this.data;
            }

            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("Data");
                }

                if (value.Length > 1024)
                {
                    throw new ArgumentOutOfRangeException("Data");
                }

                this.data = value;
            }
        }
    }

    /// <summary>
    /// FL-netログ情報
    /// </summary>
    /// <remarks>
    /// FL-netに定義されているログ情報を扱うクラスです。
    /// </remarks>
    public class LogInfo
    {
        /// <summary>
        /// ログ情報の領域を指定して、LogInfoクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="logInfo">ログ情報(仕様通りに128個(512バイト)のデータが格納された領域)</param>
        /// <exception cref="System.ArgumentNullException">引数がnull</exception>
        public LogInfo(uint[] logInfo)
        {
            if (logInfo == null)
            {
                throw new ArgumentNullException("loginfo");
            }

            this.Log = logInfo;
        }

        /// <summary>
        /// ログデータの配列表現。
        /// </summary>
        public uint[] Log { get; private set; }

        /// <summary>
        /// 通算ソケット部送信回数
        /// </summary>
        public uint SendCount
        {
            get { return this.Log[0]; }
        }

        /// <summary>
        /// 通算ソケット部送信エラー回数
        /// </summary>
        public uint SendError
        {
            get { return this.Log[1]; }
        }

        /// <summary>
        /// イーサネット送信エラー回数
        /// </summary>
        public uint EtherSendError
        {
            get { return this.Log[2]; } 
        }

        /// <summary>
        /// 送受信に関するデータ(予約3)
        /// </summary>
        public uint SendRecvReserved3
        {
            get { return this.Log[3]; }
        }

        /// <summary>
        /// 送受信に関するデータ(予約4)
        /// </summary>
        public uint SendRecvReserved4
        {
            get { return this.Log[4]; }
        }

        /// <summary>
        /// 送受信に関するデータ(予約5)
        /// </summary>
        public uint SendRecvReserved5
        {
            get { return this.Log[5]; }
        }

        /// <summary>
        /// 通算ソケット部受信回数
        /// </summary>
        public uint RecvCount
        {
            get { return this.Log[6]; }
        }

        /// <summary>
        /// 通算ソケット部受信エラー回数
        /// </summary>
        public uint RecvError
        {
            get { return this.Log[7]; }
        }

        /// <summary>
        /// イーサネット受信エラー回数
        /// </summary>
        public uint EtherRecvError
        {
            get { return this.Log[8]; }
        }

        /// <summary>
        /// 送受信に関するデータ(予約9)
        /// </summary>
        public uint SendRecvReserved9
        {
            get { return this.Log[9]; }
        }

        /// <summary>
        /// 送受信に関するデータ(予約10)
        /// </summary>
        public uint SendRecvReserved10
        { 
            get { return this.Log[10]; }
        }

        /// <summary>
        /// 送受信に関するデータ(予約11)
        /// </summary>
        public uint SendRecvReserved11
        {
            get { return this.Log[11]; }
        }

        /* --------------------------------------------------------------*/

        /// <summary>
        /// トークン送信回数
        /// </summary>
        public uint FrameSendToken
        {
            get { return this.Log[12]; }
        }

        /// <summary>
        /// サイクリックフレーム送信回数
        /// </summary>
        public uint FrameSendCyclic
        {
            get { return this.Log[13]; }
        }

        /// <summary>
        /// 1対1メッセージ送信回数
        /// </summary>
        public uint FrameSend1_1Message
        {
            get { return this.Log[14]; }
        }

        /// <summary>
        /// 1対nメッセージ送信回数
        /// </summary>
        public uint FrameSend1_NMessage
        {
            get { return this.Log[15]; }
        }

        /// <summary>
        /// フレームの種類に関するデータ(予約4)
        /// </summary>
        public uint FrameReserved4
        {
            get { return this.Log[16]; }
        }

        /// <summary>
        /// フレームの種類に関するデータ(予約5)
        /// </summary>
        public uint FrameReserved5
        {
            get { return this.Log[17]; }
        }

        /// <summary>
        /// トークン受信回数
        /// </summary>
        public uint FrameRecvToken
        {
            get { return this.Log[18]; }
        }

        /// <summary>
        /// サイクリックフレーム受信回数
        /// </summary>
        public uint FrameRecvCyclic
        {
            get { return this.Log[19]; }
        }

        /// <summary>
        /// 1対1メッセージ受信回数
        /// </summary>
        public uint FrameRecv1_1Message
        {
            get { return this.Log[20]; }
        }

        /// <summary>
        /// 1対nメッセージ受信回数
        /// </summary>
        public uint FrameRecv1_NMessage
        {
            get { return this.Log[21]; }
        }

        /// <summary>
        /// フレームの種類に関するデータ(予約10)
        /// </summary>
        public uint FrameReserved10
        {
            get { return this.Log[22]; }
        }

        /// <summary>
        /// フレームの種類に関するデータ(予約11)
        /// </summary>
        public uint FrameReserved11
        {
            get { return this.Log[23]; }
        }

        /* --------------------------------------------------------------*/

        /// <summary>
        /// サイクリック伝送受信エラー回数
        /// </summary>
        public uint ErrorCyclicTotal
        {
            get { return this.Log[24]; }
        }

        /// <summary>
        /// サイクリックアドレスサイズエラー回数
        /// </summary>
        public uint ErrorCyclicAddressSize
        {
            get { return this.Log[25]; }
        }

        /// <summary>
        /// サイクリックCBNエラー回数
        /// </summary>
        public uint ErrorCyclicCBN
        {
            get { return this.Log[26]; }
        }

        /// <summary>
        /// サイクリックTBNエラー回数
        /// </summary>
        public uint ErrorCyclicTBN
        {
            get { return this.Log[27]; }
        }

        /// <summary>
        /// サイクリックBSIZEエラー回数
        /// </summary>
        public uint ErrorCyclicBSize
        {
            get { return this.Log[28]; }
        }

        /// <summary>
        /// サイクリック伝送に関するデータ(予約5)
        /// </summary>
        public uint ErrorCyclicReserved5
        {
            get { return this.Log[29]; }
        }

        /// <summary>
        /// サイクリック伝送に関するデータ(予約6)
        /// </summary>
        public uint ErrorCyclicReserved6
        {
            get { return this.Log[30]; }
        }

        /// <summary>
        /// サイクリック伝送に関するデータ(予約7)
        /// </summary>
        public uint ErrorCyclicReserved7
        {
            get { return this.Log[31]; }
        }

        /// <summary>
        /// サイクリック伝送に関するデータ(予約8)
        /// </summary>
        public uint ErrorCyclicReserved8
        {
            get { return this.Log[32]; }
        }

        /// <summary>
        /// サイクリック伝送に関するデータ(予約9)
        /// </summary>
        public uint ErrorCyclicReserved9
        {
            get { return this.Log[33]; }
        }

        /// <summary>
        /// サイクリック伝送に関するデータ(予約10)
        /// </summary>
        public uint ErrorCyclicReserved10
        {
            get { return this.Log[34]; }
        }

        /// <summary>
        /// サイクリック伝送に関するデータ(予約11)
        /// </summary>
        public uint ErrorCyclicReserved11
        {
            get { return this.Log[35]; }
        }

        /* --------------------------------------------------------------*/

        /// <summary>
        /// メッセージ伝送再送回数
        /// </summary>
        public uint ErrorMessageRetry
        {
            get { return this.Log[36]; }
        }

        /// <summary>
        /// メッセージ伝送再送オーバ回数
        /// </summary>
        public uint ErrorMessageRetryOver
        {
            get { return this.Log[37]; }
        }

        /// <summary>
        /// メッセージ伝送に関するデータ(予約2)
        /// </summary>
        public uint ErrorMessageReserved2
        {
            get { return this.Log[38]; }
        }

        /// <summary>
        /// メッセージ伝送に関するデータ(予約3)
        /// </summary>
        public uint ErrorMessageReserved3
        {
            get { return this.Log[39]; }
        }

        /// <summary>
        /// メッセージ伝送に関するデータ(予約4)
        /// </summary>
        public uint ErrorMessageReserved4
        {
            get { return this.Log[40]; }
        }

        /// <summary>
        /// メッセージ伝送に関するデータ(予約5)
        /// </summary>
        public uint ErrorMessageReserved5
        {
            get { return this.Log[41]; }
        }

        /// <summary>
        /// メッセージ伝送に関するデータ(予約6)
        /// </summary>
        public uint ErrorMessageReserved6
        {
            get { return this.Log[42]; }
        }

        /// <summary>
        /// メッセージ伝送受信エラー回数
        /// </summary>
        public uint ErrorMessageRecvTotal
        {
            get { return this.Log[43]; }
        }

        /// <summary>
        /// メッセージ連番バージョンエラー回数
        /// </summary>
        public uint ErrorMessageTuban
        {
            get { return this.Log[44]; }
        }

        /// <summary>
        /// メッセージ連番再送認識回数
        /// </summary>
        public uint ErrorMessageVersion
        {
            get { return this.Log[45]; }
        }

        /// <summary>
        /// メッセージ伝送に関するデータ(予約10)
        /// </summary>
        public uint ErrorMessageReserved10
        {
            get { return this.Log[46]; }
        }

        /// <summary>
        /// メッセージ伝送に関するデータ(予約11)
        /// </summary>
        public uint ErrorMessageReserved11
        {
            get { return this.Log[47]; }
        }

        /* --------------------------------------------------------------*/

        /// <summary>
        /// ACKエラー回数
        /// </summary>
        public uint ErrorAckTotal
        {
            get { return this.Log[48]; }
        }

        /// <summary>
        /// ACK通番バージョンエラー回数
        /// </summary>
        public uint ErrorAckVersion
        {
            get { return this.Log[49]; }
        }

        /// <summary>
        /// ACK通番番号エラー回数
        /// </summary>
        public uint ErrorAckTuban
        {
            get { return this.Log[50]; }
        }

        /// <summary>
        /// ACKノード番号エラー回数
        /// </summary>
        public uint ErrorAckNodeNum
        {
            get { return this.Log[51]; }
        }

        /// <summary>
        /// ACK TCDエラー回数
        /// </summary>
        public uint ErrorAckTCD
        {
            get { return this.Log[52]; }
        }

        /// <summary>
        /// ACK関連に関するデータ(予約5)
        /// </summary>
        public uint ErrorAckReserved5
        {
            get { return this.Log[53]; }
        }

        /// <summary>
        /// ACK関連に関するデータ(予約6)
        /// </summary>
        public uint ErrorAckReserved6
        {
            get { return this.Log[54]; }
        }

        /// <summary>
        /// ACK関連に関するデータ(予約7)
        /// </summary>
        public uint ErrorAckReserved7
        {
            get { return this.Log[55]; }
        }

        /// <summary>
        /// ACK関連に関するデータ(予約8)
        /// </summary>
        public uint ErrorAckReserved8
        {
            get { return this.Log[56]; }
        }

        /// <summary>
        /// ACK関連に関するデータ(予約9)
        /// </summary>
        public uint ErrorAckReserved9
        {
            get { return this.Log[57]; }
        }

        /// <summary>
        /// ACK関連に関するデータ(予約10)
        /// </summary>
        public uint ErrorAckReserved10
        {
            get { return this.Log[58]; }
        }

        /// <summary>
        /// ACK関連に関するデータ(予約11)
        /// </summary>
        public uint ErrorAckReserved11
        {
            get { return this.Log[59]; }
        }

        /* --------------------------------------------------------------*/

        /// <summary>
        /// トークン多重化認識回数
        /// </summary>
        public uint ErrorTokenTajuka
        {
            get { return this.Log[60]; }
        }

        /// <summary>
        /// トークン破棄回数
        /// </summary>
        public uint ErrorTokenHaki
        {
            get { return this.Log[61]; }
        }

        /// <summary>
        /// トークン再発行回数
        /// </summary>
        public uint ErrorTokenSaihakko
        {
            get { return this.Log[62]; }
        }

        /// <summary>
        /// トークン関連に関するデータ(予約3)
        /// </summary>
        public uint ErrorTokenReserved3
        {
            get { return this.Log[63]; }
        }

        /// <summary>
        /// トークン関連に関するデータ(予約4)
        /// </summary>
        public uint ErrorTokenReserved4
        {
            get { return this.Log[64]; }
        }

        /// <summary>
        /// トークン関連に関するデータ(予約5)
        /// </summary>
        public uint ErrorTokenReserved5
        {
            get { return this.Log[65]; }
        }

        /// <summary>
        /// トークン保持タイムアウト回数
        /// </summary>
        public uint ErrorTokenHojiTimeout
        {
            get { return this.Log[66]; }
        }

        /// <summary>
        /// トークン監視タイムアウト回数
        /// </summary>
        public uint ErrorTokenKanshiTimeout
        {
            get { return this.Log[67]; }
        }

        /// <summary>
        /// トークン関連に関するデータ(予約8)
        /// </summary>
        public uint ErrorTokenReserved8
        {
            get { return this.Log[68]; }
        }

        /// <summary>
        /// トークン関連に関するデータ(予約9)
        /// </summary>
        public uint ErrorTokenReserved9
        {
            get { return this.Log[69]; }
        }

        /// <summary>
        /// トークン関連に関するデータ(予約10)
        /// </summary>
        public uint ErrorTokenReserved10
        {
            get { return this.Log[70]; }
        }

        /// <summary>
        /// トークン関連に関するデータ(予約11)
        /// </summary>
        public uint ErrorTokenReserved11
        {
            get { return this.Log[71]; }
        }

        /* --------------------------------------------------------------*/

        /// <summary>
        /// 通算稼働時間
        /// </summary>
        public uint StatusTotalTime
        {
            get { return this.Log[72]; }
        }

        /// <summary>
        /// フレーム待ち状態回数
        /// </summary>
        public uint StatusFrameWait
        {
            get { return this.Log[73]; }
        }

        /// <summary>
        /// 加入回数
        /// </summary>
        public uint StatusKanyu
        {
            get { return this.Log[74]; }
        }

        /// <summary>
        /// 自己離脱回数
        /// </summary>
        public uint StatusJikoRidatsu
        {
            get { return this.Log[75]; }
        }

        /// <summary>
        /// スキップによる離脱回数
        /// </summary>
        public uint StatusSkipRidatsu
        {
            get { return this.Log[76]; }
        }

        /// <summary>
        /// 他ノード離脱認識回数
        /// </summary>
        public uint StatusOtherRidatsu
        {
            get { return this.Log[77]; }
        }

        /// <summary>
        /// ステータスに関するデータ(予約6)
        /// </summary>
        public uint StatusReserved6
        {
            get { return this.Log[78]; }
        }

        /// <summary>
        /// ステータスに関するデータ(予約7)
        /// </summary>
        public uint StatusReserved7
        {
            get { return this.Log[79]; }
        }

        /// <summary>
        /// ステータスに関するデータ(予約8)
        /// </summary>
        public uint StatusReserved8
        {
            get { return this.Log[80]; }
        }

        /// <summary>
        /// ステータスに関するデータ(予約9)
        /// </summary>
        public uint StatusReserved9
        {
            get { return this.Log[81]; }
        }

        /// <summary>
        /// ステータスに関するデータ(予約10)
        /// </summary>
        public uint StatusReserved10
        {
            get { return this.Log[82]; }
        }

        /// <summary>
        /// ステータスに関するデータ(予約11)
        /// </summary>
        public uint StatusReserved11
        {
            get { return this.Log[83]; }
        }

        /* --------------------------------------------------------------*/
        
        /// <summary>
        /// 参加認識ノード一覧0
        /// </summary>
        public uint Status2Node0
        {
            get { return this.Log[84]; }
        }

        /// <summary>
        /// 参加認識ノード一覧1
        /// </summary>
        public uint Status2Node1
        {
            get { return this.Log[85]; }
        }

        /// <summary>
        /// 参加認識ノード一覧2
        /// </summary>
        public uint Status2Node2
        {
            get { return this.Log[86]; }
        }

        /// <summary>
        /// 参加認識ノード一覧3
        /// </summary>
        public uint Status2Node3
        {
            get { return this.Log[87]; }
        }

        /// <summary>
        /// 参加認識ノード一覧4
        /// </summary>
        public uint Status2Node4
        {
            get { return this.Log[88]; }
        }

        /// <summary>
        /// 参加認識ノード一覧5
        /// </summary>
        public uint Status2Node5
        {
            get { return this.Log[89]; }
        }

        /// <summary>
        /// 参加認識ノード一覧6
        /// </summary>
        public uint Status2Node6
        {
            get { return this.Log[90]; } 
        }

        /// <summary>
        /// 参加認識ノード一覧7
        /// </summary>
        public uint Status2Node7
        {
            get { return this.Log[91]; }
        }

        /// <summary>
        /// ステータスに関するデータ2(予約8)
        /// </summary>
        public uint Status2Reserved8
        {
            get { return this.Log[92]; }
        }

        /// <summary>
        /// ステータスに関するデータ2(予約9)
        /// </summary>
        public uint Status2Reserved9
        {
            get { return this.Log[93]; }
        }

        /// <summary>
        /// ステータスに関するデータ2(予約10)
        /// </summary>
        public uint Status2Reserved10
        {
            get { return this.Log[94]; }
        }

        /// <summary>
        /// ステータスに関するデータ2(予約11)
        /// </summary>
        public uint Status2Reserved11
        {
            get { return this.Log[95]; }
        }

        /* --------------------------------------------------------------*/

        /// <summary>
        /// その他予約(0)
        /// </summary>
        public uint MiscReserved0
        {
            get { return this.Log[96]; }
        }

        /// <summary>
        /// その他予約(1)
        /// </summary>
        public uint MiscReserved1
        {
            get { return this.Log[97]; }
        }

        /// <summary>
        /// その他予約(2) 
        /// </summary>
        public uint MiscReserved2
        {
            get { return this.Log[98]; }
        }

        /// <summary>
        /// その他予約(3) 
        /// </summary>
        public uint MiscReserved3
        {
            get { return this.Log[99]; }
        }

        /// <summary>
        /// その他予約(4) 
        /// </summary>
        public uint MiscReserved4
        {
            get { return this.Log[100]; }
        }

        /// <summary>
        /// その他予約(5) 
        /// </summary>
        public uint MiscReserved5
        {
            get { return this.Log[101]; }
        }

        /// <summary>
        /// その他予約(6) 
        /// </summary>
        public uint MiscReserved6
        {
            get { return this.Log[102]; }
        }

        /// <summary>
        /// その他予約(7) 
        /// </summary>
        public uint MiscReserved7
        {
            get { return this.Log[103]; }
        }

        /// <summary>
        /// その他予約(8) 
        /// </summary>
        public uint MiscReserved8
        {
            get { return this.Log[104]; }
        }

        /// <summary>
        /// その他予約(9) 
        /// </summary>
        public uint MiscReserved9
        {
            get { return this.Log[105]; }
        }

        /// <summary>
        /// その他予約(10) 
        /// </summary>
        public uint MiscReserved10
        {
            get { return this.Log[106]; }
        }

        /// <summary>
        /// その他予約(11) 
        /// </summary>
        public uint MiscReserved11
        {
            get { return this.Log[107]; }
        }

        /// <summary>
        /// その他予約(12) 
        /// </summary>
        public uint MiscReserved12
        {
            get { return this.Log[108]; }
        }

        /// <summary>
        /// その他予約(13) 
        /// </summary>
        public uint MiscReserved13
        {
            get { return this.Log[109]; }
        }

        /// <summary>
        /// その他予約(14) 
        /// </summary>
        public uint MiscReserved14
        {
            get { return this.Log[110]; }
        }

        /// <summary>
        /// その他予約(15) 
        /// </summary>
        public uint MiscReserved15
        {
            get { return this.Log[111]; }
        }

        /* --------------------------------------------------------------*/

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約0)
        /// </summary>
        public uint VenderReserved0
        {
            get { return this.Log[112]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約1)
        /// </summary>
        public uint VenderReserved1
        {
            get { return this.Log[113]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約2)
        /// </summary>
        public uint VenderReserved2
        {
            get { return this.Log[114]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約3)
        /// </summary>
        public uint VenderReserved3
        {
            get { return this.Log[115]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約4)
        /// </summary>
        public uint VenderReserved4
        {
            get { return this.Log[116]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約5)
        /// </summary>
        public uint VenderReserved5
        {
            get { return this.Log[117]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約6)
        /// </summary>
        public uint VenderReserved6
        {
            get { return this.Log[118]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約7)
        /// </summary>
        public uint VenderReserved7
        {
            get { return this.Log[119]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約8)
        /// </summary>
        public uint VenderReserved8
        {
            get { return this.Log[120]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約9)
        /// </summary>
        public uint VenderReserved9
        {
            get { return this.Log[121]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約10)
        /// </summary>
        public uint VenderReserved10
        {
            get { return this.Log[122]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約11)
        /// </summary>
        public uint VenderReserved11
        {
            get { return this.Log[123]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約12)
        /// </summary>
        public uint VenderReserved12
        {
            get { return this.Log[124]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約13)
        /// </summary>
        public uint VenderReserved13
        {
            get { return this.Log[125]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約14)
        /// </summary>
        public uint VenderReserved14
        {
            get { return this.Log[126]; }
        }

        /// <summary>
        /// 製造業者定義可能領域に関するデータ(予約15)
        /// </summary>
        public uint VenderReserved15
        {
            get { return this.Log[127]; }
        }
    }

    /// <summary>
    /// FL-net汎用のクラス群
    /// </summary>
    /// <remarks>
    /// FL-PCIに限定せず、FL-netを使用するのに一般的に有用なクラス群です。
    /// </remarks>
    [System.Runtime.CompilerServices.CompilerGeneratedAttribute]
    internal class NamespaceDoc
    {
    }
}
