﻿//-----------------------------------------------------------------------
// <copyright file="FLPCI.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Net;
using CenturySystems.FLNet;

namespace CenturySystems.FLPCI
{
    /// <summary>
    /// 機種情報
    /// </summary>
    public enum FLPCIModel : byte
    {
        /// <summary>
        /// FL-PCI/V2 (FL-net Ver.1)
        /// </summary>
        FLPCIV2_FLNetV1 = 0,

        /// <summary>
        /// FL-PCI/V2 (FL-net Ver.2)
        /// </summary>
        FLPCIV2_FLNetV2 = 10,

        /// <summary>
        /// FL-PCI/V2-100 or 100L (FL-net Ver.2)
        /// </summary>
        FLPCIV2_100 = 20,

        /// <summary>
        /// FL-PCI/V2-100 or 100L (FL-net Ver.3)
        /// </summary>
        FLPCIV3_100 = 30,
                                 
        /// 情報なし
        /// </summary>
        None = 255,
    }

    /// <summary>
    /// 通信速度
    /// </summary>
    public enum Speed : uint
    {
        /// <summary>
        /// 10Mbps
        /// </summary>
        _10Mbps = 0,

        /// <summary>
        /// 100Mbps
        /// </summary>
        _100Mbps = 1,

        /// <summary>
        /// オートネゴシエーション設定
        /// </summary>
        AutoNegotiation = 2,
    }

    /// <summary>
    /// 全二重/半二重
    /// </summary>
    public enum Duplex : uint
    {
        /// <summary>
        /// 半二重
        /// </summary>
        HalfDuplex = 0,

        /// <summary>
        /// 全二重
        /// </summary>
        FullDuplex = 1,
    }

    /// <summary>
    /// ノードの初期設定情報
    /// </summary>
    /// <remarks>
    /// ノードの初期設定をする際に必要となる情報です。
    /// </remarks>
    public class NodeInitInfo
    {
        /// <summary>
        /// コモンメモリ領域1先頭アドレス(0-511)
        /// </summary>
        private ushort common1Address;

        /// <summary>
        /// コモンメモリ領域1サイズ(0-512)
        /// </summary>
        private ushort common1Size;

        /// <summary>
        /// コモンメモリ領域2先頭アドレス(0-8191)
        /// </summary>
        private ushort common2Address;

        /// <summary>
        /// コモンメモリ領域2サイズ(0-8192)
        /// </summary>
        private ushort common2Size;

        /// <summary>
        /// トークン監視タイムアウト値(1-255)
        /// </summary>
        private byte tokenWatchTimeout;

        /// <summary>
        /// 最小フレーム間隔(0-50)
        /// </summary>
        private byte minFrameintervalTime;

        /// <summary>
        /// ノード名(ASCII10文字以内で任意)
        /// </summary>
        private string nodeName;

        /// <summary>
        /// NodeInitInfoクラスの新しいインスタンスをデフォルト値で初期化します。
        /// </summary>
        /// <remarks>アドレス・サイズは0、トークン監視タイムアウト時間は1、最小フレーム間隔は0、ノード名は空文字列となります。</remarks>
        public NodeInitInfo()
        {
            // 0やnull以外のデフォルト値は以下の通り
            this.TokenWatchTimeout = 1;
            this.NodeName = string.Empty;
        }

        /// <summary>
        /// NodeInitInfoクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="common1Address">コモン領域1先頭アドレス(0-511)</param>
        /// <param name="common1Size">コモン領域1サイズ(0-512)</param>
        /// <param name="common2Address">コモン領域2先頭アドレス(0-8191)</param>
        /// <param name="common2Size">コモン領域2サイズ(0-8192)</param>
        /// <param name="tokenWatchTimeout">トークン監視タイムアウト時間(1-255)</param>
        /// <param name="minFrameintervalTime">最小フレーム間隔(0-50)</param>
        /// <param name="nodeName">ノード名(ASCII 10文字以内)</param>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        /// <exception cref="System.ArgumentNullException">ノード名がnull</exception>
        /// <exception cref="System.ArgumentOutOfRangeException">ノード名がASCIIで10文字を超える</exception>
        public NodeInitInfo(
            ushort common1Address,
            ushort common1Size,
            ushort common2Address,
            ushort common2Size,
            byte tokenWatchTimeout,
            byte minFrameintervalTime,
            string nodeName)
        {
            this.Common1Address = common1Address;
            this.Common1Size = common1Size;
            this.Common2Address = common2Address;
            this.Common2Size = common2Size;
            this.TokenWatchTimeout = tokenWatchTimeout;
            this.MinFrameintervalTime = minFrameintervalTime;
            this.NodeName = nodeName;
        }

        /// <summary>
        /// コモンメモリ領域1先頭アドレス(0-511)
        /// </summary>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        public ushort Common1Address
        {
            get
            {
                return this.common1Address;
            }

            set
            {
                FLNetUtil.CheckAddress(1, value);
                this.common1Address = value;
            }
        }
        
        /// <summary>
        /// コモンメモリ領域1サイズ(0-512)
        /// </summary>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        public ushort Common1Size
        {
            get
            {
                return this.common1Size;
            }

            set
            {
                FLNetUtil.CheckSize(1, value);
                this.common1Size = value;
            }
        }
        
        /// <summary>
        /// コモンメモリ領域2先頭アドレス(0-8191)
        /// </summary>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        public ushort Common2Address
        {
            get
            {
                return this.common2Address;
            }

            set
            {
                FLNetUtil.CheckAddress(2, value);
                this.common2Address = value;
            }
        }
        
        /// <summary>
        /// コモンメモリ領域2サイズ(0-8192)
        /// </summary>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        public ushort Common2Size
        {
            get
            {
                return this.common2Size;
            }

            set
            {
                FLNetUtil.CheckSize(2, value);
                this.common2Size = value;
            }
        }
        
        /// <summary>
        /// トークン監視タイムアウト値(1-255)
        /// </summary>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        public byte TokenWatchTimeout
        {
            get
            {
                return this.tokenWatchTimeout;
            }

            set
            {
                if ((value < 1) || (value > FLPCIClass.TokenWatchMax))
                {
                    throw new ArgumentOutOfRangeException("TokenWatchTimeout");
                }

                this.tokenWatchTimeout = value;
            }
        }
        
        /// <summary>
        /// 最小フレーム間隔(0-50)
        /// </summary>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        public byte MinFrameintervalTime
        {
            get
            {
                return this.minFrameintervalTime;
            }

            set
            {
                if (value > FLPCIClass.MFTMax)
                {
                    throw new ArgumentOutOfRangeException("MinFrameintervalTime");
                }

                this.minFrameintervalTime = value;
            }
        }
        
        /// <summary>
        /// ノード名(ASCII10文字以内で任意)
        /// </summary>
        /// <exception cref="System.ArgumentNullException">ノード名がnull</exception>
        /// <exception cref="System.ArgumentOutOfRangeException">ノード名がASCIIで10文字を超える</exception>
        public string NodeName
        {
            get
            {
                return this.nodeName;
            }

            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("NodeName");
                }

                var asciiEnc = new System.Text.ASCIIEncoding();
                if (asciiEnc.GetByteCount(value) > 10)
                {
                    throw new ArgumentOutOfRangeException("NodeName");
                }

                this.nodeName = value;
            }
        }
    }

    /// <summary>
    /// メッセージの送信完了状態を表すクラス
    /// </summary>
    public class MessageEnd
    {
        /// <summary>
        /// ノードNo. (1-254or255(ブロードキャスト)
        /// </summary>
        private byte nodeNumber;

        /// <summary>
        /// MessageEndクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="node">ノードNo. (1-254or255)</param>
        /// <param name="tcd">トランザクションコード</param>
        internal MessageEnd(byte node, ushort tcd)
        {
            FLNetUtil.CheckNodeNumber(node, true);
            this.NodeNumber = node;
            this.TCD = tcd;
        }

        /// <summary>
        /// 送信完了したメッセージの送信先ノードNo.
        /// </summary>
        /// <exception cref="System.ArgumentOutOfRangeException">値が範囲外</exception>
        public byte NodeNumber
        {
            get
            {
                return this.nodeNumber;
            }

            private set
            {
                FLNetUtil.CheckNodeNumber(value, true);
                this.nodeNumber = value;
            }
        }

        /// <summary>
        /// 送信完了したメッセージのトランザクションコード
        /// </summary>
        public ushort TCD { get; private set; } // 範囲チェックはない
    }

    /// <summary>
    /// ノードの現在の情報
    /// </summary>
    public class NodeInfo : NodeInitInfo
    {
        /// <summary>
        /// 上位層の状態
        /// </summary>
        private UPLStatus uplStatus;

        /// <summary>
        /// リンクの状態
        /// </summary>
        private FLPCILinkStatus linkStatus;

        /// <summary>
        /// 自ノードの状態
        /// </summary>
        private MyStatus myStatus;

        /// <summary>
        /// ベンダ名
        /// </summary>
        private string vendorName;

        /// <summary>
        /// メーカ名
        /// </summary>
        private string makerName;

        /// <summary>
        /// NodeInfoクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="nodeInf">DLLから返ってきたノード情報</param>
        internal NodeInfo(_NodeInfo nodeInf)
        {
            // nodeInfはstructなのでnullになることはない
            this.Join = nodeInf.sanka != 0;
            if (this.Join)
            {
                this.Common1Address = nodeInf.cmn1_addr;
                this.Common1Size = nodeInf.cmn1_size;
                this.Common2Address = nodeInf.cmn2_addr;
                this.Common2Size = nodeInf.cmn2_size;
                this.TokenWatchTimeout = nodeInf.tw;
                this.MinFrameintervalTime = nodeInf.mft;
                this.NodeName = nodeInf.node;

                this.UPLStatus = FLPCIClass.CreateUPLStatus(nodeInf.uls);
                this.AllowableRefleshCycleTime = nodeInf.rct;
                this.LinkStatus = new FLPCILinkStatus(nodeInf.link_status);
                this.MyStatus = new MyStatus(nodeInf.my_status);
                this.VendorName = nodeInf.vender;
                this.MakerName = nodeInf.maker;
            }
        }

        /// <summary>
        /// FL-netに参加しているかどうか。この属性がfalseの場合は、このクラスの他の属性の値は正しくない。
        /// </summary>
        public bool Join { get; private set; }

        /// <summary>
        /// 上位層のステータス
        /// </summary>
        /// <exception cref="System.ArgumentNullException">値がnull</exception>
        public UPLStatus UPLStatus
        {
            get
            {
                return this.uplStatus;
            }

            private set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("UPLStatus");
                }

                this.uplStatus = value;
            }
        }

        /// <summary>
        /// リフレッシュサイクル許容時間
        /// </summary>
        public ushort AllowableRefleshCycleTime { get; private set; }

        /// <summary>
        /// リンクの状態
        /// </summary>
        /// <exception cref="System.ArgumentNullException">値がnull</exception>
        public FLPCILinkStatus LinkStatus
        {
            get
            {
                return this.linkStatus;
            }

            private set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("LinkStatus");
                }

                this.linkStatus = value;
            }
        }

        /// <summary>
        /// 自ノードの状態
        /// </summary>
        /// <exception cref="System.ArgumentNullException">値がnull</exception>
        public MyStatus MyStatus
        {
            get
            {
                return this.myStatus;
            }

            private set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("MyStatus");
                }

                this.myStatus = value;
            }
        }

        /// <summary>
        /// ベンダ名
        /// </summary>
        /// <exception cref="System.ArgumentNullException">値がnull</exception>
        /// <exception cref="System.ArgumentOutOfRangeException">文字列の長さがASCIIで10文字を超える</exception>
        public string VendorName
        {
            get
            {
                return this.vendorName;
            }

            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("VendorName");
                }

                var asciiEnc = new System.Text.ASCIIEncoding();
                if (asciiEnc.GetByteCount(value) > 10)
                {
                    throw new ArgumentOutOfRangeException("VendorName");
                }

                this.vendorName = value;
            }
        }
        
        /// <summary>
        /// メーカー名
        /// </summary>
        /// <exception cref="System.ArgumentNullException">値がnull</exception>
        /// <exception cref="System.ArgumentOutOfRangeException">文字列の長さがASCIIで10文字を超える</exception>
        public string MakerName
        {
            get
            {
                return this.makerName;
            }

            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("MakerName");
                }

                var asciiEnc = new System.Text.ASCIIEncoding();
                if (asciiEnc.GetByteCount(value) > 10)
                {
                    throw new ArgumentOutOfRangeException("MakerName");
                }

                this.makerName = value;
            }
        }
    }

    /// <summary>
    /// FL-PCIで使用するリンク状態。
    /// </summary>
    public class FLPCILinkStatus : LinkStatus
    {
        /// <summary>
        /// FLPCILinkStatusクラスの新しいインスタンスをデフォルト値(全てfalse)で初期化します。
        /// </summary>
        internal FLPCILinkStatus()
        { 
        }

        /// <summary>
        /// FLPCILinkStatusクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="uplError">上位層動作信号エラー</param>
        /// <param name="enableCommonMemoryData">コモンメモリ・データ有効通知</param>
        /// <param name="finishCommonMemoryConfig">コモンメモリのアドレス・サイズ設定完了</param>
        /// <param name="duplicateAddress">アドレス多重検知</param>
        /// <param name="disableComm">通信無効検知</param>
        /// <param name="join">参加/離脱</param>
        internal FLPCILinkStatus(
            bool uplError,
            bool enableCommonMemoryData,
            bool finishCommonMemoryConfig,
            bool duplicateAddress,
            bool disableComm,
            bool join)
            : base(uplError, enableCommonMemoryData, finishCommonMemoryConfig, duplicateAddress)
        {
            this.DisableComm = disableComm;
            this.Join = join;
        }

        /// <summary>
        /// リンク状態を表すbyte型の数値から、FLPCILinkStatusクラスの新しいインスタンスを初期化します。
        /// DLL呼び出しの際に使用します。
        /// </summary>
        /// <param name="linkStatusByte">リンク状態を表す数値(DLL仕様書の3.11を参照)</param>
        internal FLPCILinkStatus(byte linkStatusByte)
        {
            this.DuplicateAddress = (linkStatusByte & 0x80) != 0;
            this.FinishCommonMemoryConfig = (linkStatusByte & 0x40) != 0;
            this.EnableCommonMemoryData = (linkStatusByte & 0x20) != 0;
            this.UplError = (linkStatusByte & 0x10) != 0;
            this.DisableComm = (linkStatusByte & 0x02) != 0;
            this.Join = (linkStatusByte & 0x01) != 0;
        }

        /// <summary>
        /// FL-netに参加しているかどうか
        /// </summary>
        public bool Join { get; set; }

        /// <summary>
        /// 通信無効検知あり
        /// </summary>
        public bool DisableComm { get; set; }
    }

    /// <summary>
    /// 自ノードの状態
    /// </summary>
    public class MyStatus
    {
        /// <summary>
        /// 新しいMyStatusクラスのインスタンスを初期化します。
        /// </summary>
        /// <param name="myStatusByte">DLLから返ってきた自ノードの状態</param>
        internal MyStatus(byte myStatusByte)
        {
            this.DuplicateNodeNumber = (myStatusByte & 0x80) != 0;
            this.OccurTokenWatchTimeout = (myStatusByte & 0x40) != 0;
            this.UnableJoinNetwork = (myStatusByte & 0x20) != 0;
            this.DuplicateCommon2Address = (myStatusByte & 0x10) != 0;
            this.DuplicateCommon1Address = (myStatusByte & 0x08) != 0;
            this.OccurParameterError = (myStatusByte & 0x04) != 0;
            this.FinishNodeInit = (myStatusByte & 0x02) != 0;
        }

        /// <summary>
        /// ノード初期化完了
        /// </summary>
        public bool FinishNodeInit { get; private set; }

        /// <summary>
        /// ノード初期化でパラメータエラー発生
        /// </summary>
        public bool OccurParameterError { get; private set; }

        /// <summary>
        /// コモンメモリ領域1アドレス多重化検知
        /// </summary>
        public bool DuplicateCommon1Address { get; private set; }

        /// <summary>
        /// コモンメモリ領域2アドレス多重化検知
        /// </summary>
        public bool DuplicateCommon2Address { get; private set; }

        /// <summary>
        /// ネットワークに加入できず受信待ち状態
        /// </summary>
        public bool UnableJoinNetwork { get; private set; }

        /// <summary>
        /// トークン監視タイムアウトエラー
        /// </summary>
        public bool OccurTokenWatchTimeout { get; private set; }

        /// <summary>
        /// 自ノードのノード番号多重化検知
        /// </summary>
        public bool DuplicateNodeNumber { get; private set; }
    }

    /// <summary>
    /// リフレッシュサイクル、リンクの状態
    /// </summary>
    public class NetInfo
    {
        /// <summary>
        /// NetInfoクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="netInfo">DLLから返ってきたリフレッシュサイクル、リンクの状態</param>
        internal NetInfo(_NetInfo netInfo)
        {
            // _netInfoは値型なのでnullにはならない
            this.RCT = netInfo.rct;
            this.RCM = netInfo.rcm;
            this.RCMMax = netInfo.rcm_max;
            this.RCMMin = netInfo.rcm_min;
            this.LinkStatus = new FLPCILinkStatus(netInfo.link_status);
        }

        /// <summary>
        /// リフレッシュサイクル設定値
        /// </summary>
        public ushort RCT { get; private set; }

        /// <summary>
        /// リフレッシュサイクル測定値(現在)
        /// </summary>
        public ushort RCM { get; private set; }

        /// <summary>
        /// リフレッシュサイクル測定値(最大)
        /// </summary>
        public ushort RCMMax { get; private set; }

        /// <summary>
        /// リフレッシュサイクル測定値(最小)
        /// </summary>
        public ushort RCMMin { get; private set; }

        /// <summary>
        /// リンクの状態
        /// </summary>
        public FLPCILinkStatus LinkStatus { get; private set; }
    }

    /// <summary>
    /// 重大エラーの情報
    /// </summary>
    public class ErrInfo
    {
        /// <summary>
        /// エラーメッセージ
        /// </summary>
        private string message;

        /// <summary>
        /// ErrInfoクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="errInfo">DLLから返ってきた重大エラーの情報</param>
        internal ErrInfo(_ErrInfo errInfo)
        {
            // _ErrInfoは値型なのでnullにはならない
            this.Code = errInfo.err_code;
            this.Message = errInfo.err_msg;
        }

        /// <summary>
        /// エラーコード(詳細はDLL仕様書3.16を参照)
        /// </summary>
        public ushort Code { get; private set; }

        /// <summary>
        /// エラーメッセージ
        /// </summary>
        /// <exception cref="System.ArgumentNullException">値がnull</exception>
        public string Message
        {
            get
            {
                return this.message;
            }

            private set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("Message");
                }

                this.message = value;
            }
        }
    }

    /// <summary>
    /// 上位層の状態(ULS)とリンクの状態(LKS)
    /// </summary>
    public class ULSInfo
    {
        /// <summary>
        /// 上位層の状態
        /// </summary>
        public readonly UPLStatus UPLStatus;

        /// <summary>
        /// リンクの状態
        /// </summary>
        public readonly FLPCILinkStatus LinkStatus;

        /// <summary>
        /// ULSInfoクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="uls">DLLから返ってきた上位層の状態</param>
        /// <param name="lks">DLLから返ってきたリンクの状態</param>
        private ULSInfo(ushort uls, byte lks)
        {
            this.UPLStatus = FLPCIClass.CreateUPLStatus(uls);
            this.LinkStatus = new FLPCILinkStatus(lks);
        }

        /// <summary>
        /// DLLから返ってきた上位層の状態とリンクの状態の組から、ULSInfoクラスの配列を作成します。
        /// </summary>
        /// <param name="ulsInfo">DLLから返ってきた上位層の状態とリンクの状態(DLL仕様書 3.18参照)</param>
        /// <returns>ULSInfoクラスの配列([0]→ノード1の情報)</returns>
        internal static ULSInfo[] CreateULSInfoArray(_UlsInfo ulsInfo)
        {
            var returnValue = new ULSInfo[FLNetUtil.NodeNumberMax];
            for (int i = 0; i < returnValue.Length; i++)
            {
                returnValue[i] = new ULSInfo(ulsInfo.uls[i], ulsInfo.lks[i]);
            }

            return returnValue;
        }
    }

    /// <summary>
    /// ファームウェアのバージョン、機種情報
    /// </summary>
    public class FirmInfo
    {
        /// <summary>
        /// バージョン情報(10=Ver. 1.0)
        /// </summary>
        public readonly byte Version;

        /// <summary>
        /// モデル情報
        /// </summary>
        public readonly FLPCIModel Model;

        /// <summary>
        /// FirmInfoクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="version">DLLから返ってきたバージョン・機種情報</param>
        internal FirmInfo(ushort version)
        {
            this.Version = (byte)(version & 0x00ff);
            var modelNumber = (byte)((version & 0xff00) >> 8);
            if (!FLPCIModel.IsDefined(typeof(FLPCIModel), modelNumber))
            {
                throw new ArgumentOutOfRangeException("Model number");
            }

            this.Model = (FLPCIModel)Enum.ToObject(typeof(FLPCIModel), modelNumber);
        }
    }

    /// <summary>
    /// イーサネット通信の設定情報
    /// </summary>
    public class SpeedInfo
    {
        /// <summary>
        /// 通信速度
        /// </summary>
        public readonly Speed Speed;

        /// <summary>
        /// 全二重/半二重
        /// </summary>
        public readonly Duplex Duplex;

        /// <summary>
        /// SpeedInfoクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="speed">DLLから返ってきた速度情報</param>
        /// <param name="duplex">DLLから返ってきた全二重・半二重情報</param>
        internal SpeedInfo(uint speed, uint duplex)
        {
            if (!Speed.IsDefined(typeof(Speed), speed))
            {
                throw new ArgumentOutOfRangeException("speed");
            }

            this.Speed = (Speed)Enum.ToObject(typeof(Speed), speed);

            if (!Duplex.IsDefined(typeof(Duplex), duplex))
            {
                throw new ArgumentOutOfRangeException("duplex");
            }

            this.Duplex = (Duplex)Enum.ToObject(typeof(Duplex), duplex);
        }
    }

    /// <summary>
    /// 自ノードがFL-netに参加していないときに発生する例外。
    /// </summary>
    /// <remarks>この例外が発生した時は、自ノードの参加を待ってから再度操作を実行してください。</remarks>
    public class NotJoinException : Exception
    {
        /// <summary>
        /// NotJoinExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        internal NotJoinException()
        { 
        }
    }

    /// <summary>
    /// 指定したノードがFL-netに参加していないときに発生する例外。
    /// </summary>
    /// <remarks>この例外が発生した時は、指定したノードの状態を確認するか、しばらく経ってから再度操作を実行してください。</remarks>
    public class NodeNotJoinException : Exception
    {
        /// <summary>
        /// 参加していないノード番号
        /// </summary>
        public readonly byte NodeNumber;

        /// <summary>
        /// NodeNotJoinExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="node">参加していないノード番号</param>
        internal NodeNotJoinException(byte node)
        {
            this.NodeNumber = node;
        }
    }

    /// <summary>
    /// メッセージ送信時に失敗した場合の例外
    /// </summary>
    /// <remarks>原因は「送信先ノードの離脱」「送信先ノードからの3回リトライ後のACK未達」のいずれかです。</remarks>
    public class FailWriteMessageException : Exception
    {
        /// <summary>
        /// FailWriteMessageExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        internal FailWriteMessageException()
        {
        }
    }

    /// <summary>
    /// メッセージ送信時にバッファフルの場合の例外
    /// </summary>
    /// <remarks>この例外が発生した時は、しばらく経ってから再度操作を実行してください。</remarks>
    public class BufferFullException : Exception
    {
        /// <summary>
        /// BufferFullExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        internal BufferFullException()
        {
        }
    }

    /// <summary>
    /// メッセージ送信が未完のときに発生する例外
    /// </summary>
    /// <remarks>この例外が発生した時は、宛先ノードの状態を確認するか、しばらく経ってから再度操作を実行してください。</remarks>
    public class MessageNotSendYetException : Exception
    {
        /// <summary>
        /// MessageNotSendYetExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        internal MessageNotSendYetException()
        {
        }
    }

    /// <summary>
    /// コンストラクタ(初期化あり)を呼んだ時にボードが既に初期化されていた場合の例外
    /// </summary>
    /// <remarks>この例外が発生した時は、コンストラクタ(初期化なし)の使用を検討してください。</remarks>
    public class AlreadyInitializedException : Exception
    {
        /// <summary>
        /// 既に初期化されていたボード
        /// </summary>
        public readonly byte BoardNumber;

        /// <summary>
        /// AlreadyInitializedExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="boardNumber">ボード番号</param>
        internal AlreadyInitializedException(byte boardNumber)
        {
            this.BoardNumber = boardNumber;
        }
    }

    /// <summary>
    /// コンストラクタ(初期化なし)を呼んだ時にボードが初期化されていなかった場合の例外
    /// </summary>
    /// <remarks>この例外が発生した時は、コンストラクタ(初期化あり)の使用を検討してください。</remarks>
    public class NotInitializedException : Exception
    {
        /// <summary>
        /// 初期化されていなかったボード
        /// </summary>
        public readonly byte BoardNumber;

        /// <summary>
        /// NotInitializedExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="boardNumber">ボード番号</param>
        internal NotInitializedException(byte boardNumber)
        {
            this.BoardNumber = boardNumber;
        }
    }

    /// <summary>
    /// コンストラクタを呼んだ時にボードのデバイスが全て使用されていた場合の例外
    /// </summary>
    /// <remarks>この例外が発生した時は、どれかのデバイスをクローズしてください(他のプロセスで使用しているデバイスも含めて)</remarks>
    public class UpperLimitException : Exception
    {
        /// <summary>
        /// 上限に達したボード
        /// </summary>
        public readonly byte BoardNumber;

        /// <summary>
        /// UpperLimitExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="boardNumber">ボード番号</param>
        internal UpperLimitException(byte boardNumber)
        {
            this.BoardNumber = boardNumber;
        }
    }

    /// <summary>
    /// FLPCI.GetSpeed()を呼んだときにケーブル外れで設定が未完だった場合の例外
    /// </summary>
    /// <remarks>この例外が発生した時は、ケーブルを確認してください。</remarks>
    public class CableException : Exception
    {
        /// <summary>
        /// CableExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        internal CableException()
        {
        }
    }

    /// <summary>
    /// ハードウェア障害例外
    /// </summary>
    /// <remarks>この例外が発生した時は、弊社にご連絡ください。</remarks>
    public class HardwareException : Exception
    {
        /// <summary>
        /// エラー番号
        /// </summary>
        public readonly ushort ErrorNumber;

        /// <summary>
        /// HardwareExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="errorNumber">エラー番号</param>
        internal HardwareException(ushort errorNumber)
        {
            this.ErrorNumber = errorNumber;
        }

        /// <summary>
        /// 指定されたエラーメッセージを使用して、HardwareExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="errorNumber">エラー番号</param>
        /// <param name="message">エラーメッセージ</param>
        internal HardwareException(ushort errorNumber, string message)
            : base(message)
        {
            this.ErrorNumber = errorNumber;
        }
    }

    /// <summary>
    /// このAPIの内部エラー
    /// </summary>
    public class InternalException : Exception
    {
        /// <summary>
        /// InternalExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        internal InternalException()
        {
        }

        /// <summary>
        /// 指定されたエラーメッセージを使用して、InternalExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="message">エラーメッセージ</param>
        internal InternalException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// 指定されたエラーメッセージと、この例外の原因である内部例外への参照を使用して、
        /// InternalExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="message">エラーメッセージ</param>
        /// <param name="inner">原因の例外</param>
        internal InternalException(string message, Exception inner)
            : base(message, inner)
        {
        }
    }

    /// <summary>
    /// DLL呼び出しで発生したエラーを表す例外
    /// </summary>
    public class DLLException : InternalException
    {
        /// <summary>
        /// エラーが発生したDLLの関数名
        /// </summary>
        public readonly string Function;

        /// <summary>
        /// 指定されたエラーメッセージを使用して、DLLExceptionクラスの新しいインスタンスを初期化します。
        /// </summary>
        /// <param name="function">関数名</param>
        /// <param name="message">エラーメッセージ</param>
        internal DLLException(string function, string message)
            : base(message)
        {
            this.Function = function;
        }
    }

    /// <summary>
    /// FL-PCIクラス
    /// </summary>
    public partial class FLPCIClass
    {
        /// <summary>
        /// トークン監視タイムアウト設定値上限
        /// </summary>
        public const byte TokenWatchMax = 255;

        /// <summary>
        /// 最小フレーム間隔設定値上限
        /// </summary>
        public const byte MFTMax = 50;

        /// <summary>
        /// APIのバージョン番号
        /// </summary>
        private const ushort APIVersionNumber = 10;

        /// <summary>
        /// 1台のPCに接続できるFL-PCIの最大枚数
        /// </summary>
        private const byte BoardMaxNumber = 2;

        /// <summary>
        /// 1枚のFL-PCIで制御できるデバイスの最大数
        /// </summary>
        private const byte DeviceMaxNumber = 2;

        /// <summary>
        /// ボード番号
        /// </summary>
        private readonly byte boardNumber;

        /// <summary>
        /// デバイスがオープンされているかどうか
        /// </summary>
        private bool isOpened = false;

        /// <summary>
        /// ボードのうち何番目のデバイスを使っているか
        /// </summary>
        private byte boardDeviceNumber;

        /// <summary>
        /// FL-PCIボードを初期化し、デバイスをオープンする。
        /// </summary>
        /// <param name="ipAddress">自ノードIPアドレス</param>
        /// <param name="boardNumber">ボードNo. (1-2)</param>
        /// <param name="speed">通信速度</param>
        /// <param name="nodeInitInfo">ノードの初期設定パラメータ</param>
        /// <exception cref="System.ArgumentNullException">引数がnull</exception>
        /// <exception cref="System.ArgumentOutOfRangeException">引数が範囲外、IPアドレスがIPv4以外</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        /// <exception cref="System.TimeoutException">タイムアウト(最大4秒)</exception>
        /// <exception cref="CenturySystems.FLPCI.AlreadyInitializedException">既にボードが初期化されている</exception>
        /// <remarks>
        /// DLLで使用するデバイス番号は、指定されたボードのうち、未使用の番号を自動的に取得します(指定できません)。
        /// </remarks>
        public FLPCIClass(
            IPAddress ipAddress,
            byte boardNumber,
            Speed speed,
            NodeInitInfo nodeInitInfo)
        {
            if (ipAddress == null)
            {
                throw new ArgumentNullException("ipAddress");
            }

            if (nodeInitInfo == null)
            {
                throw new ArgumentNullException("nodeInitInfo");
            }

            if (ipAddress.AddressFamily != System.Net.Sockets.AddressFamily.InterNetwork)
            {
                throw new ArgumentOutOfRangeException("ipAddress");
            }

            FLPCIClass.CheckBoardNumber(boardNumber);
            this.boardNumber = boardNumber;

            for (byte b = 0; b <= DeviceMaxNumber; b++)
            {  // デバイス番号の最後まで行ったら例外を発生させるために <= にしている 
                if (b == DeviceMaxNumber)
                {
                    throw new UpperLimitException(boardNumber);
                }

                this.boardDeviceNumber = b;
                var initInfo = new _InitInfo(ipAddress, nodeInitInfo, this.DeviceNumber);

                // Set_SpeedはFL_Initializeの前に呼ぶ必要がある
                ushort rc = NativeMethods.Set_Speed((uint)speed, this.DeviceNumber);
                if (rc != 0)
                {
                    this.ThrowDLLException(rc, "Set_Speed");
                }

                rc = NativeMethods.FL_Initialize(ref initInfo);
                if (rc == 0)
                {
                    // オープン成功
                    this.isOpened = true;
                    break;
                }
                else
                {
                    switch (rc)
                    {
                        case 16:
                            // このデバイス番号は既にオープン済み。デバイス番号を更新して再度実行。
                            continue;
                        case 18:
                            // このボードは既に初期化済み。
                            throw new AlreadyInitializedException(this.boardNumber);
                        default:
                            this.ThrowDLLException(rc, "FL_Initialize");
                            throw new InternalException(); // 上で必ず例外発生するけど一応
                    }
                }
            }
        }

        /// <summary>
        /// コンストラクタ(ボードを初期化しない)。
        /// </summary>
        /// <param name="boardNumber">ボード番号</param>
        /// <exception cref="CenturySystems.FLPCI.NotInitializedException">ボードが初期化されていない</exception>
        public FLPCIClass(byte boardNumber)
        {
            FLPCIClass.CheckBoardNumber(boardNumber);
            this.boardNumber = boardNumber;

            for (byte b = 0; b <= DeviceMaxNumber; b++)
            {  // デバイス番号の最後まで行ったら例外を発生させるために <= にしている 
                if (b == DeviceMaxNumber)
                {
                    throw new UpperLimitException(boardNumber);
                }

                this.boardDeviceNumber = b;
                ushort rc = NativeMethods.DeviceOpen(this.DeviceNumber);
                if (rc == 0)
                {
                    // オープン成功
                    this.isOpened = true;
                    break;
                }
                else
                {
                    switch (rc)
                    {
                        case 16:
                            // このデバイス番号は既にオープン済み。デバイス番号を更新して再度実行。
                            continue;
                        case 17:
                            throw new NotInitializedException(this.boardNumber);
                        default:
                            this.ThrowDLLException(rc, "DeviceOpen");
                            throw new InternalException(); // 上で必ず例外発生するけど一応
                    }
                }
            }
        }

        /// <summary>
        /// デストラクタ(呼ばれることを想定してはいけない。保険用)
        /// </summary>
        ~FLPCIClass()
        {
            if (this.isOpened)
            {
                try
                {
                    this.CloseDevice(false);
                }
                catch (Exception)
                {
                }
            }
        }

        /// <summary>
        /// デバイス番号 (DLLで使用)
        /// </summary>
        private ushort DeviceNumber
        {
            get
            {
                return (ushort)(((this.boardNumber - 1) * DeviceMaxNumber) + this.boardDeviceNumber);
            }
        }

        /// <summary>
        /// API, DLLのバージョン情報を取得する。
        /// </summary>
        /// <returns>[0]→DLLのバージョン [1]→APIのバージョン</returns>
        public static ushort[] GetAPIVersion()
        {
            ushort[] returnValue = { NativeMethods.Dll_Version(), FLPCIClass.APIVersionNumber };
            return returnValue;
        }

        /// <summary>
        /// ボードの全てのデバイスをcloseし、ボードをリセットし、FLPCIクラスの状態を初期状態に戻します。
        /// </summary>
        /// <param name="boardNumber">ボードNo. (1-2)</param>
        /// <exception cref="System.ArgumentOutOfRangeException">ボードNo.が範囲外</exception>
        /// <remarks>
        /// InternalExceptionが発生した場合は、このメソッドの使用を検討してください。
        /// このメソッドを呼んだ場合、これ以前から存在するFLPCIクラスのインスタンスの動作は保証されません。
        /// ボード番号を間違った場合、プログラムの動作は保証されません。
        /// </remarks>
        public static void ResetAllDevice(byte boardNumber)
        {
            FLPCIClass.CheckBoardNumber(boardNumber);
            for (byte i = 0; i < DeviceMaxNumber; i++)
            {
                try
                {
                    NativeMethods.DeviceClose((ushort)(((boardNumber - 1) * DeviceMaxNumber) + i), (ushort)1);　// 返り値は無視
                }
                catch (Exception)
                {
                }
            }
        }

        /// <summary>
        /// ボード番号が範囲内にあるかチェック
        /// </summary>
        /// <param name="boardNumber">ボード番号(1-2)</param>
        /// <exception cref="System.ArgumentOutOfRangeException">ボード番号が範囲外</exception>
        public static void CheckBoardNumber(byte boardNumber)
        {
            if ((boardNumber == 0) || (boardNumber > BoardMaxNumber))
            {
                throw new ArgumentOutOfRangeException("boardNumber");
            }
        }

        /// <summary>
        /// デバイスをクローズする。
        /// </summary>
        /// <param name="reset">FL-PCIをRESET状態にするか否か</param>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        public void CloseDevice(
            bool reset)
        {
            this.isOpened = false;
            var rc = NativeMethods.DeviceClose(this.DeviceNumber, reset ? (ushort)1 : (ushort)0);
            if (rc != 0)
            {
                switch (rc)
                {
                    case 15:
                        throw new DLLException("DeviceClose", "Device close error. deviceNumber = " + this.DeviceNumber);
                    default:
                        this.ThrowDLLException(rc, "DeviceClose");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }
        }

        /// <summary>
        /// コンストラクタで設定した自ノードコモンメモリ領域の先頭から、サイクリックデータを書き込む。
        /// </summary>
        /// <param name="buff1Data">領域1のサイクリックデータ</param>
        /// <param name="buff2Data">領域2のサイクリックデータ</param>
        /// <exception cref="System.ArgumentNullException">引数がnull</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        /// <exception cref="CenturySystems.FLPCI.BufferFullException">バッファフル</exception>
        /// <remarks>
        /// 指定するサイクリックデータのサイズが自ノードのコモン領域サイズより小さい場合の動作は保証されません。
        /// </remarks>
        public void WriteCyclicAll(
            ushort[] buff1Data,
            ushort[] buff2Data)
        {
            if (buff1Data == null)
            {
                throw new ArgumentNullException("buff1Data");
            }

            if (buff2Data == null)
            {
                throw new ArgumentNullException("buff2Data");
            }

            ushort rc = NativeMethods.WriteCyclicALL(buff1Data, buff2Data, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    case 3:
                        throw new BufferFullException();
                    default:
                        this.ThrowDLLException(rc, "WriteCyclicALL");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }
        }

        /// <summary>
        /// コンストラクタで設定した自ノードコモンメモリ領域の先頭を0とする相対アドレスに、サイクリックデータを書き込む。
        /// </summary>
        /// <param name="commonNumber">コモン領域番号(1-2)</param>
        /// <param name="offset">アドレスオフセット(0-511または0-8191)</param>
        /// <param name="data">サイクリックデータ</param>
        /// <exception cref="System.ArgumentNullException">引数がnull</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        /// <exception cref="CenturySystems.FLPCI.BufferFullException">バッファフル</exception>
        /// <remarks>
        /// (アドレスオフセット + 指定するサイクリックデータのサイズ)が、自ノードのコモンメモリ領域の大きさより小さい場合、そのデータだけが書かれる。
        /// 大きい場合は、余りのデータは無視される。
        /// </remarks>
        public void WriteCyclicRelative(
            byte commonNumber,
            ushort offset,
            ushort[] data)
        {
            FLNetUtil.CheckCommonNumber(commonNumber);
            FLNetUtil.CheckAddress(commonNumber, offset);
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }

            var length = (uint)data.Length;

            //// lengthが大きすぎる場合はエラーにならないようにDLL呼び出し前に補正(例外にはしない)
            if (length > FLNetUtil.CommonSizeMax[commonNumber])
            {
                length = FLNetUtil.CommonSizeMax[commonNumber];
            }

            ushort rc = NativeMethods.WriteCyclicRelative(commonNumber, offset, (ushort)length, data, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    case 3:
                        throw new BufferFullException();
                    default:
                        this.ThrowDLLException(rc, "WriteCyclicRelative");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }
        }

        /// <summary>
        /// 指定された領域からサイクリックデータを読み出す。
        /// </summary>
        /// <param name="commonNumber">コモン領域番号(1-2)</param>
        /// <param name="address">読み出し元アドレス(0-511または0-8191)</param>
        /// <param name="size">読み出しワードサイズ(1-512または1-8192)</param>
        /// <returns>読み出したサイクリックデータ</returns>
        /// <exception cref="System.ArgumentOutOfRangeException">引数が範囲外</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        /// <remarks>
        /// (address + size) がコモン領域のサイズ(512, 8192)を超えた場合は、返り値の要素数が切り詰められます。
        /// (例: コモン領域1, address=510, size=10と指定した場合、返り値の要素数は2になります) 
        /// </remarks>
        public ushort[] ReadCyclicAbsolute(
            byte commonNumber,
            ushort address,
            ushort size)
        {
            FLNetUtil.CheckCommonNumber(commonNumber);
            FLNetUtil.CheckAddress(commonNumber, address);
            FLNetUtil.CheckSize(commonNumber, size);

            if (address + size > FLNetUtil.CommonSizeMax[commonNumber])
            {
                size = (ushort)(FLNetUtil.CommonSizeMax[commonNumber] - address);
            }

            var returnValue = new ushort[size];
            ushort rc = NativeMethods.ReadCyclicAbsolute(commonNumber, address, size, returnValue, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    default:
                        this.ThrowDLLException(rc, "ReadCyclicAbsolute");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }

            return returnValue;
        }
        
        /// <summary>
        /// 指定されたノードが占有する領域先頭を0とする相対アドレスから、サイクリックデータを読み出す。
        /// </summary>
        /// <param name="nodeNumber">ノード番号(1～254)</param>
        /// <param name="commonNumber">コモン領域番号(1～2)</param>
        /// <param name="offset">アドレスオフセット(0～511または0～8191)</param>
        /// <param name="size">読み出しワードサイズ(1～512または1～8192)</param>
        /// <returns>読み出したサイクリックデータ</returns>
        /// <exception cref="System.ArgumentOutOfRangeException">引数が範囲外</exception>
        /// <exception cref="CenturySystems.FLPCI.NodeNotJoinException">指定ノードがFL-netに参加していない</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        /// <remarks>
        /// (指定ノードの先頭アドレス + offset + size) がコモン領域のサイズ(512, 8192)を超えた場合は、返り値の要素数が切り詰められます。
        /// (例: コモン領域1, 先頭アドレス=500, offset=10, size=10と指定した場合、返り値の要素数は2になります) 
        /// </remarks>
        public ushort[] ReadCyclicRelative(
            byte nodeNumber,
            byte commonNumber,
            ushort offset,
            ushort size)
        {
            FLNetUtil.CheckNodeNumber(nodeNumber, false);
            FLNetUtil.CheckCommonNumber(commonNumber);
            FLNetUtil.CheckAddress(commonNumber, offset);
            FLNetUtil.CheckSize(commonNumber, size);

            // 指定された要素の先頭アドレスを得ます
            var nodeInf = this.GetNodeInfo(nodeNumber);
            if (nodeInf.Join == false)
            {
                throw new NodeNotJoinException(nodeNumber);
            }

            ushort address;
            switch (commonNumber)
            {
                case 1:
                    address = nodeInf.Common1Address;
                    break;
                case 2:
                    address = nodeInf.Common2Address;
                    break;
                default:
                    throw new InternalException();
            }

            // サイズを切り詰める必要があれば切り詰めます
            if (address + offset + size > FLNetUtil.CommonSizeMax[commonNumber])
            {
                size = (ushort)(FLNetUtil.CommonSizeMax[commonNumber] - address - offset);
            }

            var returnValue = new ushort[size];
            ushort rc = NativeMethods.ReadCyclicRelative(nodeNumber, commonNumber, offset, size, returnValue, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    case 5:
                        throw new NodeNotJoinException(nodeNumber);
                    default:
                        this.ThrowDLLException(rc, "ReadCyclicRelative");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }

            return returnValue;
        }

        /// <summary>
        /// 指定されたノードが占有する全てのサイクリックデータを読み出す。
        /// </summary>
        /// <param name="nodeNumber">ノード番号(1～254)</param>
        /// <returns>サイクリックデータの各領域の配列([0]→常にnull, [1]→領域1, [2]→領域2)</returns>
        /// <exception cref="System.ArgumentOutOfRangeException">引数が範囲外</exception>
        /// <exception cref="CenturySystems.FLPCI.NodeNotJoinException">指定ノードがFL-netに参加していない</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        /// <remarks>
        /// 返り値の配列(領域1, 領域2)の大きさは、指定ノードのコモンメモリ領域サイズです。
        /// </remarks>
        public ushort[][] ReadCyclicAll(
            byte nodeNumber)
        {
            FLNetUtil.CheckNodeNumber(nodeNumber, false);

            // 指定された要素のサイズを得ます。
            var nodeInf = this.GetNodeInfo(nodeNumber);
            if (nodeInf.Join == false)
            {
                throw new NodeNotJoinException(nodeNumber);
            }

            var buf1 = new ushort[nodeInf.Common1Size];
            var buf2 = new ushort[nodeInf.Common2Size];
            ushort rc = NativeMethods.ReadCyclicALL(nodeNumber, buf1, buf2, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    case 5:
                        throw new NodeNotJoinException(nodeNumber);
                    default:
                        this.ThrowDLLException(rc, "ReadCyclicALL");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }

            return new ushort[][] { null, buf1, buf2 };
        }

        /// <summary>
        /// 指定されたメッセージを送信する。
        /// </summary>
        /// <param name="messageInfo">送信メッセージ</param>
        /// <param name="waitToFinish">送信完了を待つか否か</param>
        /// <exception cref="System.ArgumentNullException">引数がnull</exception>
        /// <exception cref="CenturySystems.FLPCI.NotJoinException">自ノードが離脱状態</exception>
        /// <exception cref="CenturySystems.FLPCI.BufferFullException">バッファフル</exception>
        /// <exception cref="CenturySystems.FLPCI.FailWriteMessageException">送信失敗</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        /// <exception cref="System.TimeoutException">タイムアウト(ACKが返る前に自ノードが離脱)</exception>
        /// <remarks>バッファフルの場合は、アプリケーション側で再送信を行ってください。</remarks>
        public void WriteMessage(
            MessageInfo messageInfo,
            bool waitToFinish)
        {
            if (messageInfo == null)
            {
                throw new ArgumentNullException("messageInfo");
            }

            var mi = new _MessageInfo(messageInfo);
            ushort rc = NativeMethods.WriteMessage(ref mi, messageInfo.Data, (ushort)(waitToFinish ? 1 : 0), this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    case 3:
                        throw new BufferFullException();
                    case 5:
                        throw new NotJoinException();
                    case 6:
                        throw new FailWriteMessageException();
                    default:
                        this.ThrowDLLException(rc, "WriteMessage");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }
        }

        /// <summary>
        /// WriteMessage()でメッセージ送信完了を待たなかった場合に、メッセージ送信の終了ステータスを取得する。
        /// 送信失敗・未完の時は例外を発生する。
        /// </summary>
        /// <returns>メッセージ送信の終了ステータス。</returns>
        /// <exception cref="CenturySystems.FLPCI.FailWriteMessageException">送信失敗</exception>
        /// <exception cref="CenturySystems.FLPCI.MessageNotSendYetException">送信未完</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        public MessageEnd CheckMessageEnd()
        {
            ushort node;
            ushort tcd;
            ushort rc = NativeMethods.CheckMessageEnd(out node, out tcd, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc) 
                {
                    case 6:
                        throw new FailWriteMessageException();
                    case 7:
                        throw new MessageNotSendYetException();
                    default:
                        this.ThrowDLLException(rc, "CheckMessageEnd");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }

            MessageEnd returnValue = null;
            try
            {
                returnValue = new MessageEnd((byte)node, tcd);
            }
            catch (ArgumentOutOfRangeException e)
            {
                throw new InternalException("nodeNumber", e);
            }

            return returnValue;
        }

        /// <summary>
        /// メッセージを受信する。
        /// </summary>
        /// <param name="waitForFinish">受信完了を待つか否か</param>
        /// <returns>受信したメッセージ</returns>
        /// <exception cref="System.TimeoutException">メッセージなし(受信完了を待たないとき)</exception>
        /// <exception cref="System.TimeoutException">タイムアウト(2秒)(受信完了を待つとき)</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        /// <exception cref="CenturySystems.FLPCI.InternalException">ネットワークから受信したデータが異常</exception>
        public MessageInfo ReadMessage(
            bool waitForFinish)
        {
            var messageInfo = new _MessageInfo();
            var buff = new byte[1024];
            ushort rc = NativeMethods.ReadMessage(out messageInfo, buff, (ushort)(waitForFinish ? 1 : 0), this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    case 4:
                        throw new TimeoutException();
                    default:
                        this.ThrowDLLException(rc, "ReadMessage");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }

            MessageInfo returnValue = null;
            try
            {
                returnValue = messageInfo.MakeMessageInfo(buff);
            }
            catch (ArgumentOutOfRangeException e)
            {
                // 受信データが異常。
                throw new InternalException("Received data is invalid", e);
            }

            return returnValue;
        }

        /// <summary>
        /// 指定したノードの情報を取得する。
        /// </summary>
        /// <param name="nodeNumber">ノード番号(1～254)</param>
        /// <returns>指定したノードの情報</returns>
        /// <exception cref="System.ArgumentOutOfRangeException">ノード番号が範囲外</exception>
        /// <exception cref="CenturySystems.FLPCI.NodeNotJoinException">指定したノードが参加していない</exception>
        /// <exception cref="System.TimeoutException">タイムアウト(1秒)</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        public NodeInfo GetNodeInfo(
            byte nodeNumber)
        {
            FLNetUtil.CheckNodeNumber(nodeNumber, false);

            var nodeInf = new _NodeInfo();
            ushort rc = NativeMethods.GetNodeInfo(nodeNumber, out nodeInf, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    default:
                        this.ThrowDLLException(rc, "GetNodeInfo");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }

            if (nodeInf.sanka == 0)
            {
                throw new NodeNotJoinException(nodeNumber);
            }

            NodeInfo returnValue = null;
            try
            {
                returnValue = new NodeInfo(nodeInf);
            }
            catch (ArgumentOutOfRangeException e)
            {
                // NodeInfに無効な値が入っていた場合の処理
                throw new InternalException("NodeInfo has invalid value", e);
            }

            return returnValue;
        }

        /// <summary>
        /// 現在のFL-netに参加中のノード一覧を取得する。
        /// </summary>
        /// <returns>参加しているノード番号(自ノードを除く)</returns>
        /// <exception cref="System.TimeoutException">タイムアウト(1秒)</exception>
        /// <exception cref="CenturySystems.FLPCI.NotJoinException">自ノードが離脱状態</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        public byte[] GetRingInfo()
        {
            var table = new _SankaReqTable();
            ushort rc = NativeMethods.GetRingInfo(out table, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    default:
                        this.ThrowDLLException(rc, "GetRingInfo");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }

            if (table.node_cnt == 0)
            {
                throw new NotJoinException();
            }

            var returnValue = new byte[table.node_cnt];
            Array.Copy(table.node_no, 0, returnValue, 0, table.node_cnt);
            return returnValue;
        }

        /// <summary>
        /// 現在のFL-netのリフレッシュサイクル等を取得する。
        /// </summary>
        /// <returns>リフレッシュサイクル等の情報</returns>
        /// <exception cref="System.TimeoutException">タイムアウト(1秒)</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        public NetInfo GetNetInfo()
        {
            var netInfo = new _NetInfo();
            ushort rc = NativeMethods.GetNetInfo(out netInfo, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    default:
                        this.ThrowDLLException(rc, "GetNetInfo");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }

            return new NetInfo(netInfo);
        }

        /// <summary>
        /// 自ノードのログ情報を取得する。
        /// </summary>
        /// <returns>ログ情報</returns>
        /// <exception cref="System.TimeoutException">タイムアウト(1秒)</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        public LogInfo GetLogInfo()
        {
            var logInfo = new uint[128];
            ushort rc = NativeMethods.GetLogInfo(logInfo, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    default:
                        this.ThrowDLLException(rc, "GetLogInfo");
                        throw new InternalException(); // 上で例外発生するけど一応
                }   
            }

            return new LogInfo(logInfo);
        }

        /// <summary>
        /// 上位層ステータス(ULS)を設定する。
        /// </summary>
        /// <param name="uplStatus">上位層ステータス(ULS)</param>
        /// <exception cref="System.ArgumentNullException">引数がnull</exception>
        /// <exception cref="System.TimeoutException">タイムアウト(1秒)</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        public void SetUplStatus(
            UPLStatus uplStatus)
        {
            ushort uls = UPLStatusToUShort(uplStatus);
            ushort rc = NativeMethods.SetUplStatus(uls, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    default:
                        this.ThrowDLLException(rc, "SetUplStatus");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }
        }

        /// <summary>
        /// 重大エラー情報を取得する。
        /// </summary>
        /// <returns>エラー情報</returns>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        public ErrInfo GetErrInfo()
        {
            var errInfo = new _ErrInfo();
            ushort rc = NativeMethods.GetErrInfo(out errInfo, this.DeviceNumber);
            if (rc != 0)
            {
                this.ThrowDLLException(rc, "GetErrInfo");
            }

            return new ErrInfo(errInfo);
        }

        /// <summary>
        /// 参加中のノードのULS, LKSを取得する。
        /// </summary>
        /// <returns>上位層・リンクの状態</returns>
        /// <exception cref="System.TimeoutException">タイムアウト(1秒)</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        public ULSInfo[] GetULSAll()
        {
            var ulsInfo = new _UlsInfo();
            ushort rc = NativeMethods.ReadUls_All(out ulsInfo, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    default:
                        this.ThrowDLLException(rc, "ReadUls_All");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }

            return ULSInfo.CreateULSInfoArray(ulsInfo);
        }

        /// <summary>
        /// 機種、ファームウェアバージョンを取得する。
        /// </summary>
        /// <returns>機種、ファームウェアのバージョン情報</returns>
        /// <exception cref="System.TimeoutException">タイムアウト(1秒)</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        public FirmInfo GetFirmwareVersion()
        {
            ushort version;
            ushort rc = NativeMethods.Firmware_Version(out version, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    default:
                        this.ThrowDLLException(rc, "Firmware_Version");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }

            return new FirmInfo(version);
        }

        /// <summary>
        /// 通信速度、デュープレックスモードを取得する。
        /// </summary>
        /// <returns>通信速度、デュープレックスモードの情報</returns>
        /// <exception cref="System.TimeoutException">タイムアウト(1秒)</exception>
        /// <exception cref="CenturySystems.FLPCI.DLLException">DLL呼び出しエラー</exception>
        /// <exception cref="CenturySystems.FLPCI.CableException">ケーブル外れのため設定が未完</exception>
        public SpeedInfo GetSpeed()
        {
            uint speed, duplex;
            ushort rc = NativeMethods.Get_Speed(out speed, out duplex, this.DeviceNumber);
            if (rc != 0)
            {
                switch (rc)
                {
                    case 20:
                        throw new CableException();
                    default:
                        this.ThrowDLLException(rc, "Get_Speed");
                        throw new InternalException(); // 上で例外発生するけど一応
                }
            }

            SpeedInfo returnValue;
            try
            {
                returnValue = new SpeedInfo(speed, duplex);
            }
            catch (ArgumentOutOfRangeException e)
            {
                throw new InternalException("DLL returns invalid data", e);
            }

            return returnValue;
        }

        /// <summary>
        /// 文字列化(デバッグ用)
        /// </summary>
        /// <returns>インスタンスを表す文字列</returns>
        public override string ToString()
        {
            return "FLPCI: Device " + this.DeviceNumber;
        }
    }

    /// <summary>
    /// FL-PCI用のクラス群。メインとなるクラスは FLPCIClass クラスです。
    /// </summary>
    /// <remarks>
    /// FL-PCIを使用するのに有用なクラス群です。
    /// </remarks>
    [System.Runtime.CompilerServices.CompilerGeneratedAttribute]
    internal class NamespaceDoc
    {
    }
}