﻿//-----------------------------------------------------------------------
// <copyright file="DelegateCommand.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Windows.Input;

namespace CenturySystems.WPFSample.Common
{
    /// <summary>
    /// デリゲートを受け取るICommandの実装
    /// </summary>
    public class DelegateCommand : ICommand
    {
        /// <summary>
        /// 実行する処理
        /// </summary>
        private Action execute;

        /// <summary>
        /// 実行可能どうかを返す関数
        /// </summary>
        private Func<bool> canExecute;

        /// <summary>
        /// コマンドのExecuteメソッドで実行する処理を指定してDelegateCommandのインスタンスを作成します。 
        /// </summary>
        /// <param name="execute">Executeメソッドで実行する処理</param>
        public DelegateCommand(Action execute)
            : this(execute, () => true)
        {
        }

        /// <summary>
        /// コマンドのExecuteメソッドで実行する処理とCanExecuteメソッドで実行する処理を指定して 
        /// DelegateCommandのインスタンスを作成します。 
        /// </summary>
        /// <param name="execute">Executeメソッドで実行する処理</param>
        /// <param name="canExecute">CanExecuteメソッドで実行する処理</param>
        public DelegateCommand(Action execute, Func<bool> canExecute)
        {
            if (execute == null)
            {
                throw new ArgumentNullException("execute");
            }

            if (canExecute == null)
            {
                throw new ArgumentNullException("canExecute");
            }

            this.execute = execute;
            this.canExecute = canExecute;
        }

        /// <summary>
        /// CanExecuteの結果に変更があったことを通知するイベント。 
        /// WPFのCommandManagerのRequerySuggestedイベントをラップする形で実装しています。 
        /// </summary>
        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        /// <summary>
        /// コマンドを実行します。
        /// </summary>
        public void Execute()
        {
            this.execute();
        }

        /// <summary>
        /// コマンドが実行可能な状態化どうか問い合わせます。 
        /// </summary>
        /// <returns>実行可能な場合はtrue</returns>
        public bool CanExecute()
        {
            return this.canExecute();
        }

        /// <summary>
        /// ICommand.CanExecuteの明示的な実装。CanExecuteメソッドに処理を委譲する。 
        /// </summary>
        /// <param name="parameter">パラメータ</param>
        /// <returns>実行可能かどうか</returns>
        bool ICommand.CanExecute(object parameter)
        {
            return this.canExecute();
        }

        /// <summary>
        /// ICommand.Executeの明示的な実装。Executeメソッドに処理を委譲する。 
        /// </summary>
        /// <param name="parameter">パラメータ</param>
        void ICommand.Execute(object parameter)
        {
            this.execute();
        }
    }
}
