﻿//-----------------------------------------------------------------------
// <copyright file="CommonMemory.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.Timers;
using CenturySystems.FLNet;
using CenturySystems.FLPCI;
using CenturySystems.WPFSample.Common;

namespace CenturySystems.WPFSample.Model
{
    /// <summary>
    /// コモンメモリのクラス
    /// </summary>
    public class CommonMemory : ModelBase, IDisposable
    {
        /// <summary>
        /// モデルメイン
        /// </summary>
        private ModelMain modelMain;

        /// <summary>
        /// インターバルタイマ
        /// </summary>
        private Timer intervalTimer = null;

        /// <summary>
        /// このオブジェクトのDispose()が既に呼ばれたか
        /// </summary>
        private bool disposed = false;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="main">モデルメイン</param>
        public CommonMemory(ModelMain main)
        {
            this.ModelMain = main;
            this.Memory = new ushort[3][];
        }

        /// <summary>
        /// モデルメイン
        /// </summary>
        public ModelMain ModelMain
        {
            get
            {
                return this.modelMain;
            }

            private set
            {
                this.modelMain = value;
                if (this.modelMain != null)
                {
                    // イベントを受ける設定
                    this.ModelPropertyChanged += this.OnListenerPropertyChanged;
                    PropertyChangedEventManager.AddListener(this.modelMain, this, string.Empty);
                }
            }
        }

        /// <summary>
        /// コモンメモリ領域
        /// </summary>
        public ushort[][] Memory { get; private set; }

        /// <summary>
        /// 自動更新のスタートが可能か
        /// </summary>
        public bool CanStartInterval
        {
            get
            {
                return (this.intervalTimer == null) && this.ModelMain.CanCallFLPCIMethod;
            }
        }

        /// <summary>
        /// 自動更新のストップが可能か
        /// </summary>
        public bool CanStopInterval
        {
            get
            {
                return (this.intervalTimer != null);
            }
        }

        /// <summary>
        /// 手動更新が可能か
        /// </summary>
        public bool CanUpdate
        {
            get
            {
                return (this.intervalTimer == null) && this.ModelMain.CanCallFLPCIMethod;
            }
        }

        /// <summary>
        /// 一定間隔ごとの自動更新をスタートします。
        /// </summary>
        /// <param name="interval">インターバル(ミリ秒)</param>
        public void StartInterval(int interval)
        {
            lock (this)
            {
                if (this.intervalTimer != null)
                {
                    throw new ApplicationException("this.intervalTimer is not null.");
                }

                // 最初に1回更新します。
                this.Update();

                this.intervalTimer = new Timer(interval);
                this.intervalTimer.Elapsed += new ElapsedEventHandler(this.Update);
                this.intervalTimer.Start();
            }

            this.RaisePropertyChanged("CanStartInterval");
            this.RaisePropertyChanged("CanStopInterval");
            this.RaisePropertyChanged("CanUpdate");
        }

        /// <summary>
        /// 一定間隔ごとの自動更新をストップします。
        /// </summary>
        public void StopInterval()
        {
            lock (this)
            {
                if (this.intervalTimer == null)
                {
                    return;
                }

                this.intervalTimer.Stop();
                this.intervalTimer.Dispose();
                this.intervalTimer = null;
            }

            this.RaisePropertyChanged("CanStartInterval");
            this.RaisePropertyChanged("CanStopInterval");
            this.RaisePropertyChanged("CanUpdate");
        }

        /// <summary>
        /// 1回のみ更新します。
        /// </summary>
        public void Update()
        {
            lock (this)
            {
                lock (this.ModelMain)
                {
                    if (this.ModelMain.FLPCI == null)
                    {
                        this.ModelMain.ErrorMessageToViewModel = "FLPCI is not initialized.";
                        return;
                    }

                    if (this.ModelMain.CanCallFLPCIMethod == false)
                    {
                        this.ModelMain.ErrorMessageToViewModel = "FLPCI is unavailable just now.";
                        return;
                    }

                    try
                    {
                        for (byte i = 1; i <= 2; i++)
                        {
                            this.Memory[i] = this.ModelMain.FLPCI.ReadCyclicAbsolute(i, 0, FLNetUtil.CommonSizeMax[i]);
                        }
                    }
                    catch (DLLException e)
                    {
                        this.ModelMain.ErrorMessageToViewModel = e.ToString();
                        return;
                    }
                }
            }

            this.RaisePropertyChanged("Memory");
        }

        /// <summary>
        /// オブジェクトを破棄する
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// オブジェクトを破棄する
        /// </summary>
        /// <param name="disposing">マネージドオブジェクトを破棄するか</param>
        protected virtual void Dispose(bool disposing)
        {
            if (this.disposed)
            {
                return;
            }

            if (disposing)
            {
                if (this.intervalTimer != null)
                {
                    this.intervalTimer.Dispose();
                }
            }

            this.disposed = true;
        }

        /// <summary>
        /// タイマによって更新します。
        /// </summary>
        /// <param name="sender">イベント送信者</param>
        /// <param name="e">イベント引数</param>
        private void Update(object sender, ElapsedEventArgs e)
        {
            this.Update();
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var modelMain = (ModelMain)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "CanCallFLPCIMethod":
                    // FLPCIが更新されたので、"CanStartInterval"などの状態も変わる
                    this.RaisePropertyChanged("CanStartInterval");
                    this.RaisePropertyChanged("CanUpdate");
                    break;
                default:
                    break;
            }
        }
    }
}
