﻿//-----------------------------------------------------------------------
// <copyright file="FLPCIVersion.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.ComponentModel;
using CenturySystems.FLPCI;
using CenturySystems.WPFSample.Common;

namespace CenturySystems.WPFSample.Model
{
    /// <summary>
    /// バージョン情報を表すモデル
    /// </summary>
    public class FLPCIVersion : ModelBase
    {
        /// <summary>
        /// モデルメイン
        /// </summary>
        private ModelMain modelMain;

        /// <summary>
        /// DLLのバージョン
        /// </summary>
        private ushort dllVersion;

        /// <summary>
        /// APIのバージョン
        /// </summary>
        private ushort apiVersion;

        /// <summary>
        /// FL-PCIの機種
        /// </summary>
        private FLPCIModel modelName;

        /// <summary>
        /// ファームウェアのバージョン
        /// </summary>
        private ushort firmVersion;

        /// <summary>
        /// インスタンスを初期化します。
        /// </summary>
        /// <param name="modelMain">モデルメイン</param>
        public FLPCIVersion(ModelMain modelMain)
        {
            this.ModelMain = modelMain;
            this.ModelName = FLPCIModel.None;
            this.FirmVersion = 0;

            this.UpdateAPIVersion();
        }

        /// <summary>
        /// モデルメイン
        /// </summary>
        public ModelMain ModelMain
        {
            get
            {
                return this.modelMain;
            }

            private set
            {
                this.modelMain = value;
                if (this.modelMain != null)
                {
                    // イベントを受ける設定
                    this.ModelPropertyChanged += this.OnListenerPropertyChanged;
                    PropertyChangedEventManager.AddListener(this.modelMain, this, string.Empty);
                }
            }
        }

        /// <summary>
        /// DLLのバージョン
        /// </summary>
        public ushort DLLVersion
        {
            get
            {
                return this.dllVersion;
            }

            set
            {
                this.dllVersion = value;
                this.RaisePropertyChanged("DLLVersion");
            }
        }

        /// <summary>
        /// APIのバージョン
        /// </summary>
        public ushort APIVersion
        {
            get
            {
                return this.apiVersion;
            }

            set
            {
                this.apiVersion = value;
                this.RaisePropertyChanged("APIVersion");
            }
        }

        /// <summary>
        /// 機種の種類
        /// </summary>
        public FLPCIModel ModelName
        {
            get
            {
                return this.modelName;
            }

            set
            {
                this.modelName = value;
                this.RaisePropertyChanged("ModelName");
            }
        }

        /// <summary>
        /// ファームウェアのバージョン
        /// </summary>
        public ushort FirmVersion
        {
            get
            {
                return this.firmVersion;
            }

            set
            {
                this.firmVersion = value;
                this.RaisePropertyChanged("FirmVersion");
            }
        }

        /// <summary>
        /// DLLとAPIのバージョン情報を更新します。更新した結果はプロパティに反映され、イベントが発生します。
        /// </summary>
        public void UpdateAPIVersion()
        {
            var versions = FLPCIClass.GetAPIVersion();
            this.DLLVersion = versions[0];
            this.APIVersion = versions[1];
        }

        /// <summary>
        /// ファームウェアの機種とバージョン情報を更新します。更新した結果はプロパティに反映され、イベントが発生します。
        /// </summary>
        public void UpdateFirmVersion()
        {
            if (this.modelMain.FLPCI != null)
            {
                var firmInfo = this.modelMain.FLPCI.GetFirmwareVersion();
                this.ModelName = firmInfo.Model;
                this.FirmVersion = firmInfo.Version;
            }
            else
            {
                this.ModelName = FLPCIModel.None;
                this.FirmVersion = 0;
            }
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var modelMain = (ModelMain)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "CanCallFLPCIMethod":
                    // FLPCIが更新されたので、ファームウェアのバージョン情報を更新する
                    this.UpdateFirmVersion();
                    break;
                default:
                    break;
            }
        }
    }
}
