﻿//-----------------------------------------------------------------------
// <copyright file="LogInformation.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.ComponentModel;
using CenturySystems.FLNet;
using CenturySystems.FLPCI;
using CenturySystems.WPFSample.Common;

namespace CenturySystems.WPFSample.Model
{
    /// <summary>
    /// ログ情報のモデル
    /// </summary>
    public class LogInformation : ModelBase
    {
        /// <summary>
        /// モデルメイン
        /// </summary>
        private ModelMain modelMain;

        /// <summary>
        /// ノード情報
        /// </summary>
        private LogInfo logInfo;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="modelMain">モデルメイン</param>
        public LogInformation(ModelMain modelMain)
        {
            this.ModelMain = modelMain;
        }

        /// <summary>
        /// モデルメイン
        /// </summary>
        public ModelMain ModelMain
        {
            get
            {
                return this.modelMain;
            }

            private set
            {
                this.modelMain = value;
                if (this.modelMain != null)
                {
                    // イベントを受ける設定
                    this.ModelPropertyChanged += this.OnListenerPropertyChanged;
                    PropertyChangedEventManager.AddListener(this.modelMain, this, string.Empty);
                }
            }
        }

        /// <summary>
        /// ログ情報
        /// </summary>
        public LogInfo LogInfo
        {
            get
            {
                return this.logInfo;
            }

            set
            {
                this.logInfo = value;
                this.RaisePropertyChanged("LogInfo");
            }
        }

        /// <summary>
        /// ログ情報を更新できる状態かどうか(現状はFLPCIが初期化されていればtrue)
        /// </summary>
        public bool CanUpdateLogInfo
        {
            get
            {
                if (this.modelMain == null)
                {
                    return false;
                }

                return this.modelMain.CanCallFLPCIMethod;
            }
        }

        /// <summary>
        /// ログ情報を更新します。更新した結果はプロパティに反映され、イベントが発生します。
        /// </summary>
        /// <param name="nodeNumber">ノード番号</param>
        public void UpdateLogInfo(byte nodeNumber)
        {
            lock (this)
            {
                lock (this.modelMain)
                {
                    if (this.modelMain == null)
                    {
                        throw new ApplicationException("modelMain is null");
                    }

                    if (this.modelMain.FLPCI == null)
                    {
                        this.modelMain.ErrorMessageToViewModel = "FLPCI is not initialized";
                        return;
                    }

                    LogInfo logInfo = null;
                    try
                    {
                        if (this.modelMain.NodeNumber == nodeNumber)
                        {
                            // 自分自身のログを取得する
                            logInfo = this.modelMain.FLPCI.GetLogInfo();
                        }
                        else
                        {
                            // 他ノードのログを取得する
                            var message = new MessageInfo(nodeNumber, ResultCode.Normal, (ushort)TCDCode.LogDataReadReq, 0, 0, 0, new byte[0]);
                            this.modelMain.FLPCI.WriteMessage(message, true);
                            MessageInfo logMessage = null;
                            while (true)
                            {
                                logMessage = this.modelMain.FLPCI.ReadMessage(true);
                                if (logMessage.TCD == (ushort)TCDCode.LogDataReadRsp)
                                {
                                    break;
                                }
                            }

                            if (logMessage.Data.Length != 512)
                            {
                                this.modelMain.ErrorMessageToViewModel = "Invalid log data.";
                                return;
                            }

                            var logDataOrig = logMessage.Data;
                            var logData = new uint[128];
                            for (int i = 0; i < logData.Length; i++)
                            {
                                logData[i] = (uint)((logDataOrig[i * 4 + 3] << 24) + (logDataOrig[i * 4 + 2] << 16) + (logDataOrig[i * 4 + 1] << 8) + logDataOrig[i * 4]);
                            }

                            logInfo = new LogInfo(logData);
                        }

                        this.LogInfo = logInfo;
                    }
                    catch (TimeoutException)
                    {
                        this.modelMain.ErrorMessageToViewModel = "Timeout.";
                    }
                    catch (NotJoinException)
                    {
                        this.modelMain.ErrorMessageToViewModel = "My board is not join.";
                    }
                    catch (BufferFullException)
                    {
                        this.modelMain.ErrorMessageToViewModel = "Buffer full.";
                    }
                    catch (FailWriteMessageException)
                    {
                        this.modelMain.ErrorMessageToViewModel = "Fail to write message.";
                    }
                    catch (Exception e)
                    {
                        this.modelMain.ErrorMessageToViewModel = e.ToString();
                    }
                }
            }
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var modelMain = (ModelMain)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "CanCallFLPCIMethod":
                    this.RaisePropertyChanged("CanUpdateLogInfo");
                    break;
                default:
                    break;
            }
        }
    }
}
