﻿//-----------------------------------------------------------------------
// <copyright file="Message.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Threading;
using System.Threading.Tasks;
using CenturySystems.FLNet;
using CenturySystems.FLPCI;
using CenturySystems.WPFSample.Common;

namespace CenturySystems.WPFSample.Model
{
    /// <summary>
    /// メッセージ送受信を行うモデル
    /// </summary>
    public class Message : ModelBase, IDisposable
    {
        /// <summary>
        /// モデルメイン
        /// </summary>
        public readonly ModelMain ModelMain;

        /// <summary>
        /// メッセージ送受信をスタートした時刻
        /// </summary>
        private DateTime startDateTime;

        /// <summary>
        /// 受信用タスク
        /// </summary>
        private Task receiveTask = null;

        /// <summary>
        /// キャンセルトークンの発生源
        /// </summary>
        private CancellationTokenSource tokenSource;

        /// <summary>
        /// 受信したメッセージ
        /// </summary>
        private ObservableCollection<MessageToViewModel> receivedMessage;

        /// <summary>
        /// 送信中かどうか
        /// </summary>
        private bool inSend = false;

        /// <summary>
        /// Dispose()が呼ばれたかどうか
        /// </summary>
        private bool disposed = false;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="modelMain">モデルメイン</param>
        public Message(ModelMain modelMain)
        {
            if (modelMain == null)
            {
                throw new ArgumentNullException("modelMain");
            }

            this.ModelMain = modelMain;
            
            // イベントを受ける設定
            this.ModelPropertyChanged += this.OnListenerPropertyChanged;
            PropertyChangedEventManager.AddListener(this.ModelMain, this, string.Empty);

            this.ReceivedMessage = new ObservableCollection<MessageToViewModel>();
        }

        /// <summary>
        /// メッセージ受信の開始が可能か
        /// </summary>
        public bool CanStartComm
        {
            get
            {
                return (this.ModelMain.CanCallFLPCIMethod) && (this.receiveTask == null);
            }
        }

        /// <summary>
        /// メッセージ受信の終了が可能か
        /// </summary>
        public bool CanStopComm
        {
            get
            {
                return (this.receiveTask != null);
            }
        }

        /// <summary>
        /// メッセージ送信の受付が可能か。送信動作中は受け付けない。
        /// </summary>
        public bool CanSendMessage
        {
            get
            {
                return (this.ModelMain.CanCallFLPCIMethod) && (this.InSend == false);
            }
        }

        /// <summary>
        /// 送信中かどうか
        /// </summary>
        public bool InSend
        {
            get
            {
                return this.inSend;
            }

            private set
            {
                this.inSend = value;
                this.RaisePropertyChanged("CanSendMessage");
            }
        }

        /// <summary>
        /// 受信したメッセージ
        /// </summary>
        public ObservableCollection<MessageToViewModel> ReceivedMessage
        {
            get
            {
                return this.receivedMessage;
            }

            private set
            {
                this.receivedMessage = value;
                this.RaisePropertyChanged("ReceivedMessage");
            }
        }

        /// <summary>
        /// 受信タスク
        /// </summary>
        private Task ReceiveTask
        {
            get
            {
                return this.receiveTask;
            }

            set
            {
                this.receiveTask = value;
                this.RaisePropertyChanged("CanStartComm");
                this.RaisePropertyChanged("CanStopComm");
            }
        }
        
        /// <summary>
        /// メッセージの受信を開始する
        /// </summary>
        public void StartComm()
        {
            this.ReceivedMessage.Clear();
            this.tokenSource = new CancellationTokenSource();
            var token = this.tokenSource.Token;

            this.startDateTime = DateTime.Now;
            this.ReceiveTask = Task.Factory.StartNew(() => this.RunReceive(token), token);
        }

        /// <summary>
        /// メッセージの受信を終了する
        /// </summary>
        public void StopComm()
        {
            if (this.tokenSource != null)
            {
                this.tokenSource.Cancel();
            }

            this.ReceiveTask = null;
        }

        /// <summary>
        /// メッセージを受信する。タスクとして実行。
        /// </summary>
        /// <param name="token">キャンセルトークン</param>
        public void RunReceive(CancellationToken token)
        {
            if (this.ModelMain.FLPCI == null)
            {
                return;
            }

            while (true)
            {
                if (token.IsCancellationRequested)
                {
                    return;
                }

                try
                {
                    lock (this)
                    {
                        lock (this.ModelMain)
                        {
                            var messageInfo = this.ModelMain.FLPCI.ReadMessage(false);
                            var timeSpan = DateTime.Now - this.startDateTime;
                            var message = new MessageToViewModel(timeSpan, messageInfo);
                            this.ReceivedMessage.Add(message);
                        }
                    }
                }
                catch (TimeoutException)
                {
                    // 0.1秒待つ
                    System.Threading.Thread.Sleep(100);
                }
            }
        }

        /// <summary>
        /// メッセージを送信します。
        /// </summary>
        /// <param name="message">メッセージ</param>
        public void SendMessage(MessageInfo message)
        {
            if (message == null)
            {
                throw new ArgumentNullException("message");
            }

            if (this.CanSendMessage == false)
            {
                this.ModelMain.ErrorMessageToViewModel = "cannot send message.";
                return;
            }

            bool finished = false;
            try
            {
                this.InSend = true;
                for (int i = 0; i < 5; i++)
                {
                    try
                    {
                        this.ModelMain.FLPCI.WriteMessage(message, false);
                        finished = true;
                        break;
                    }
                    catch (FailWriteMessageException e)
                    {
                        this.ModelMain.ErrorMessageToViewModel = e.ToString();
                        return;
                    }
                    catch (DLLException e)
                    {
                        this.ModelMain.ErrorMessageToViewModel = e.ToString();
                        return;
                    }
                    catch (NotJoinException)
                    {
                        // 他の例外なら1秒待って再送信
                        Thread.Sleep(1000);
                    }
                    catch (BufferFullException)
                    {
                        // 他の例外なら1秒待って再送信
                        Thread.Sleep(1000);
                    }
                    catch (TimeoutException)
                    {
                        // 他の例外なら1秒待って再送信
                        Thread.Sleep(1000);
                    }
                }
            }
            finally
            {
                this.InSend = false;
            }

            if (finished == false)
            {
                this.ModelMain.ErrorMessageToViewModel = "send failed.";
                return;
            }
        }

        /// <summary>
        /// オブジェクトを破棄する
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// オブジェクトを破棄する
        /// </summary>
        /// <param name="disposing">マネージドオブジェクトを破棄するか</param>
        protected virtual void Dispose(bool disposing)
        {
            if (this.disposed)
            {
                return;
            }

            if (disposing)
            {
                this.tokenSource.Dispose();
            }

            this.disposed = true;
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var modelMain = (ModelMain)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "CanCallFLPCIMethod":
                    // FLPCIが更新されたので、"CanStartComm"などの状態も変わる
                    this.RaisePropertyChanged("CanStartComm");
                    this.RaisePropertyChanged("CanSendMessage");
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// ビューモデルに見せるメッセージのモデル
        /// </summary>
        public class MessageToViewModel
        {
            /// <summary>
            /// コンストラクタ
            /// </summary>
            /// <param name="timeSpan">メッセージ送受信開始からの時間</param>
            /// <param name="messageInfo">メッセージ</param>
            public MessageToViewModel(TimeSpan timeSpan, MessageInfo messageInfo)
            {
                this.TimeSpan = timeSpan;
                this.MessageInfo = messageInfo;
            }

            /// <summary>
            /// メッセージ
            /// </summary>
            public MessageInfo MessageInfo { get; private set; }

            /// <summary>
            /// メッセージ送受信開始からの時間
            /// </summary>
            public TimeSpan TimeSpan { get; private set; }
        }
    }
}
