﻿//-----------------------------------------------------------------------
// <copyright file="ModelMain.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Net;
using CenturySystems.FLNet;
using CenturySystems.FLPCI;
using CenturySystems.WPFSample.Common;

namespace CenturySystems.WPFSample.Model
{
    /// <summary>
    /// モデル全てを所有するクラス
    /// </summary>
    public class ModelMain : ModelBase, IDisposable
    {
        /// <summary>
        /// バージョンのモデル
        /// </summary>
        public readonly FLPCIVersion Version;

        /// <summary>
        /// ノード情報のモデル
        /// </summary>
        public readonly NodeInformation NodeInformation;

        /// <summary>
        /// ログ情報のモデル
        /// </summary>
        public readonly LogInformation LogInformation;

        /// <summary>
        /// その他のモデル
        /// </summary>
        public readonly SetULS Other;

        /// <summary>
        /// 自ノードサイクリックのモデル
        /// </summary>
        public readonly MyCyclic MyCyclic;

        /// <summary>
        /// コモンメモリのモデル
        /// </summary>
        public readonly CommonMemory CommonMemory;

        /// <summary>
        /// ネットワーク情報のモデル
        /// </summary>
        public readonly NetworkInformation NetworkInformation;

        /// <summary>
        /// メッセージ送受信のモデル
        /// </summary>
        public readonly Message Message;

        /// <summary>
        /// FL-PCIクラス
        /// </summary>
        private FLPCIClass flpci;

        /// <summary>
        /// 色々な操作を外部から受け付けることができるかどうか
        /// </summary>
        private bool canAcceptCommand;

        /// <summary>
        /// ノードを初期化したときの情報
        /// </summary>
        private NodeInitInfo nodeInitInfo;

        /// <summary>
        /// クライアントに送るエラーメッセージ
        /// </summary>
        private string errorMessageToViewModel;

        /// <summary>
        /// Dispose()が呼ばれたか
        /// </summary>
        private bool disposed = false;

        /// <summary>
        /// モデルを構築します。
        /// </summary>
        public ModelMain()
        {
            // 個別のモデルの構築
            this.Version = new FLPCIVersion(this);
            this.NodeInformation = new NodeInformation(this);
            this.LogInformation = new LogInformation(this);
            this.Other = new SetULS(this);
            this.MyCyclic = new MyCyclic(this);
            this.CommonMemory = new CommonMemory(this);
            this.NetworkInformation = new NetworkInformation(this);
            this.Message = new Message(this);

            this.FLPCI = null;
            this.CanAcceptCommand = true;
            this.NodeInitInfo = null;
            this.ErrorMessageToViewModel = string.Empty;
        }

        /// <summary>
        /// デストラクタ
        /// </summary>
        ~ModelMain()
        {
            this.Dispose(false);
            this.Finally();
        }

        /// <summary>
        /// FL-PCIクラス
        /// </summary>
        public FLPCIClass FLPCI
        {
            get
            {
                return this.flpci;
            }

            set
            {
                this.flpci = value;
                
                // 付随するイベントの発生(このモデルがもつプロパティが更新されたというイベント)
                this.RaisePropertyChanged("CanInitializeBoard");
                this.RaisePropertyChanged("CanCallFLPCIMethod"); 
            }
        }

        /// <summary>
        /// ボードが使用されておらず、ボード初期化が可能ならtrue
        /// </summary>
        public bool CanInitializeBoard
        {
            get
            {
                return (this.FLPCI == null) && this.canAcceptCommand;
            }
        }

        /// <summary>
        /// FLPCIのインスタンスのメソッドが呼べる状態ならtrue
        /// </summary>
        public bool CanCallFLPCIMethod
        {
            get
            {
                return (this.FLPCI != null) && this.canAcceptCommand;
            }
        }

        /// <summary>
        /// ビューモデルに送るエラーメッセージ。値を代入するとイベントが発生します。
        /// </summary>
        public string ErrorMessageToViewModel
        {
            get
            {
                return this.errorMessageToViewModel;
            }

            internal set
            {
                this.errorMessageToViewModel = value;
                this.RaisePropertyChanged("ErrorMessageToViewModel");
            }
        }

        /// <summary>
        /// ノード番号
        /// </summary>
        public byte NodeNumber { get; private set; }

        /// <summary>
        /// FL-PCIを初期化したときの情報
        /// </summary>
        public NodeInitInfo NodeInitInfo
        {
            get
            {
                return this.nodeInitInfo;
            }

            private set
            {
                this.nodeInitInfo = value;
                this.RaisePropertyChanged("NodeInitInfo");
            }
        }

        /// <summary>
        /// 色々な操作を外部から受け付けることができるかどうか
        /// </summary>
        private bool CanAcceptCommand
        {
            set
            {
                this.canAcceptCommand = value;
                this.RaisePropertyChanged("CanInitializeBoard");
                this.RaisePropertyChanged("CanCallFLPCIMethod");
            }
        }

        //----Method----

        /// <summary>
        /// ボードを初期化する
        /// </summary>
        /// <param name="ipAddress">IPアドレス</param>
        /// <param name="boardNumber">ボード番号</param>
        /// <param name="speed">速度</param>
        /// <param name="nodeInitInfo">ノード初期化情報</param>
        public void InitializeBoard(IPAddress ipAddress, byte boardNumber, Speed speed, NodeInitInfo nodeInitInfo)
        {
            lock (this)
            {
                try
                {
                    this.FLPCI = new FLPCIClass(ipAddress, boardNumber, speed, nodeInitInfo);

                    this.NodeNumber = ipAddress.GetAddressBytes()[3];
                    this.NodeInitInfo = nodeInitInfo;

                    this.ErrorMessageToViewModel = "FLPCI Initialized.";
                }
                catch (Exception e)
                {
                    this.ErrorMessageToViewModel = e.ToString();
                }
            }
        }

        /// <summary>
        /// 終了時の処理
        /// </summary>
        public void Finally()
        {
            lock (this)
            {
                if (this.FLPCI != null)
                {
                    this.CloseDevice(true);
                    this.FLPCI = null;
                }
            }
        }

        /// <summary>
        /// デバイスを閉じる
        /// </summary>
        /// <param name="reset">ボードをリセットするならtrue</param>
        public void CloseDevice(bool reset)
        {
            lock (this)
            {
                this.CanAcceptCommand = false;

                // モデル内でタイマで動いている処理を全て止める
                this.CommonMemory.StopInterval();
                this.Message.StopComm();

                try
                {
                    this.FLPCI.CloseDevice(reset);
                    this.ErrorMessageToViewModel = "FLPCI closed.";
                }
                catch (Exception e)
                {
                    this.ErrorMessageToViewModel = e.ToString();
                }
                finally
                {
                    this.FLPCI = null;
                    this.NodeNumber = 0;
                    this.NodeInitInfo = null;
                }

                this.CanAcceptCommand = true;
            }
        }

        /// <summary>
        /// 全てのデバイスを閉じて初期化する。
        /// </summary>
        /// <param name="board">ボード番号</param>
        public void CloseAndResetAllDevice(byte board)
        {
            // モデル内でタイマで動いている処理を全て止める
            this.CommonMemory.StopInterval();
            this.Message.StopComm();

            FLPCIClass.ResetAllDevice(board);
            this.FLPCI = null;
            this.NodeNumber = 0;
            this.NodeInitInfo = null;

            this.ErrorMessageToViewModel = "FLPCI reseted.";
        }

        /// <summary>
        /// オブジェクトを破棄する
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// オブジェクトを破棄する
        /// </summary>
        /// <param name="disposing">マネージドオブジェクトを破棄するか</param>
        protected virtual void Dispose(bool disposing)
        {
            if (this.disposed)
            {
                return;
            }

            if (disposing)
            {
                this.CommonMemory.Dispose();
                this.Message.Dispose();
                this.NetworkInformation.Dispose();
            }

            this.disposed = true;
        }
    }
}
