﻿//-----------------------------------------------------------------------
// <copyright file="MyCyclic.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.ComponentModel;
using CenturySystems.FLPCI;
using CenturySystems.WPFSample.Common;

namespace CenturySystems.WPFSample.Model
{
    /// <summary>
    /// 自ノードサイクリックデータのモデル
    /// </summary>
    public class MyCyclic : ModelBase
    {
        /// <summary>
        /// モデルメイン
        /// </summary>
        private ModelMain modelMain;

        /// <summary>
        /// インスタンスを初期化します。
        /// </summary>
        /// <param name="modelMain">モデルメイン</param>
        public MyCyclic(ModelMain modelMain)
        {
            this.ModelMain = modelMain;
        }

        /// <summary>
        /// モデルメイン
        /// </summary>
        public ModelMain ModelMain
        {
            get
            {
                return this.modelMain;
            }

            set
            {
                this.modelMain = value;
                if (this.modelMain != null)
                {
                    // イベントを受ける設定
                    this.ModelPropertyChanged += this.OnListenerPropertyChanged;
                    PropertyChangedEventManager.AddListener(this.modelMain, this, string.Empty);
                }
            }
        }

        /// <summary>
        /// FLPCIのインスタンスのメソッドが呼べる状態ならtrue
        /// </summary>
        public bool CanCallFLPCIMethod
        {
            get
            {
                return this.modelMain.CanCallFLPCIMethod;
            }
        }

        /// <summary>
        /// ノードを初期化したときの情報
        /// </summary>
        public NodeInitInfo NodeInitInfo
        {
            get
            {
                return this.modelMain.NodeInitInfo;
            }
        }

        /// <summary>
        /// 自ノードのコモン領域にデータを書き込みます。
        /// </summary>
        /// <param name="commonNumber">コモン番号 (1-2)</param>
        /// <param name="offset">先頭からのオフセット</param>
        /// <param name="size">サイズ</param>
        /// <param name="data">データ</param>
        public void WriteCommonMemory(byte commonNumber, ushort offset, ushort size, ushort data)
        {
            var dataArray = new ushort[size];
            for (int i = 0; i < dataArray.Length; i++)
            {
                dataArray[i] = data;
            }

            lock (this.modelMain)
            {
                if (this.modelMain.CanCallFLPCIMethod == false)
                {
                    this.modelMain.ErrorMessageToViewModel = "FL-PCI is unavailavle just now.";
                    return;
                }

                try
                {
                    this.modelMain.FLPCI.WriteCyclicRelative(commonNumber, offset, dataArray);
                    this.modelMain.ErrorMessageToViewModel = "Success.";
                }
                catch (ArgumentException)
                {
                    this.modelMain.ErrorMessageToViewModel = "Invalid Argument.";
                }
                catch (DLLException e)
                {
                    this.modelMain.ErrorMessageToViewModel = e.ToString();
                }
                catch (BufferFullException)
                {
                    this.modelMain.ErrorMessageToViewModel = "Buffer full now. Please retry later.";
                }
            }
        }

        /// <summary>
        /// 自ノードの全コモン領域に同一のデータを書き込みます。
        /// </summary>
        /// <param name="data">データ</param>
        public void WriteCommonMemoryAll(ushort data)
        {
            lock (this.modelMain)
            {
                NodeInitInfo nodeInitInfo = this.modelMain.NodeInitInfo;
                if (nodeInitInfo == null)
                {
                    this.modelMain.ErrorMessageToViewModel = "FL-PCI is not initialized.";
                    return;
                }

                if (this.modelMain.CanCallFLPCIMethod == false)
                {
                    this.modelMain.ErrorMessageToViewModel = "FL-PCI is unavailavle just now.";
                    return;
                }

                var common1Size = nodeInitInfo.Common1Size;
                var common1Data = new ushort[common1Size];
                for (int i = 0; i < common1Data.Length; i++)
                {
                    common1Data[i] = data;
                }

                var common2Size = nodeInitInfo.Common2Size;
                var common2Data = new ushort[common2Size];
                for (int i = 0; i < common2Data.Length; i++)
                {
                    common2Data[i] = data;
                }

                try
                {
                    this.modelMain.FLPCI.WriteCyclicAll(common1Data, common2Data);
                    this.modelMain.ErrorMessageToViewModel = "Success.";
                }
                catch (DLLException e)
                {
                    this.modelMain.ErrorMessageToViewModel = e.ToString();
                }
                catch (BufferFullException)
                {
                    this.modelMain.ErrorMessageToViewModel = "Buffer full now. Please retry later.";
                }
            }
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            // ビューモデルにイベントを送る
            this.RaisePropertyChanged(e.PropertyName);
        }
    }
}
