﻿//-----------------------------------------------------------------------
// <copyright file="NetworkInformation.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Timers;
using CenturySystems.FLPCI;
using CenturySystems.WPFSample.Common;

namespace CenturySystems.WPFSample.Model
{
    /// <summary>
    /// ネットワーク情報のモデル
    /// </summary>
    public class NetworkInformation : ModelBase, IDisposable
    {
        /// <summary>
        /// モデルメイン
        /// </summary>
        public readonly ModelMain ModelMain;

        /// <summary>
        /// 参加中のノード
        /// </summary>
        private byte[] nodes;

        /// <summary>
        /// ネットワーク状態
        /// </summary>
        private NetInfo netInfo;

        /// <summary>
        /// 自ノードの状態
        /// </summary>
        private NodeInfo myNodeInfo;

        /// <summary>
        /// 自ノードの通信モード
        /// </summary>
        private SpeedInfo mySpeedInfo;

        /// <summary>
        /// 自動取得のためのタイマ
        /// </summary>
        private Timer intervalTimer;

        /// <summary>
        /// Dispose()が呼ばれたか
        /// </summary>
        private bool disposed = false;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="modelMain">モデルメイン</param>
        public NetworkInformation(ModelMain modelMain)
        {
            this.ModelMain = modelMain;

            // 情報の自動取得をスタート
            this.StartInterval();
        }

        /// <summary>
        /// 参加中のノード
        /// </summary>
        public byte[] Nodes
        {
            get
            {
                return this.nodes;
            }

            private set
            {
                this.nodes = value;
                this.RaisePropertyChanged("Nodes");
            }
        }

        /// <summary>
        /// ネットワーク状態
        /// </summary>
        public NetInfo NetInfo
        {
            get
            {
                return this.netInfo;
            }

            private set
            {
                this.netInfo = value;
                this.RaisePropertyChanged("NetInfo");
            }
        }

        /// <summary>
        /// 自ノードの状態
        /// </summary>
        public NodeInfo MyNodeInfo
        {
            get
            {
                return this.myNodeInfo;
            }

            private set
            {
                this.myNodeInfo = value;
                this.RaisePropertyChanged("MyNodeInfo");
            }
        }

        /// <summary>
        /// 自ノードの通信モード
        /// </summary>
        public SpeedInfo MySpeedInfo
        {
            get
            {
                return this.mySpeedInfo;
            }

            private set
            {
                this.mySpeedInfo = value;
                this.RaisePropertyChanged("MySpeedInfo");
            }
        }

        /// <summary>
        /// 情報の自動取得をスタート(デフォルト: 3秒間隔)
        /// </summary>
        public void StartInterval()
        {
            int interval = 3000;

            lock (this)
            {
                // 最初に1回更新します。
                this.Update();

                this.intervalTimer = new Timer(interval);
                this.intervalTimer.Elapsed += new ElapsedEventHandler(this.Update);
                this.intervalTimer.Start();
            }
        }

        /// <summary>
        /// 情報の自動取得をストップ
        /// </summary>
        public void StopInterval()
        {
            lock (this)
            {
                if (this.intervalTimer == null)
                {
                    return;
                }

                this.intervalTimer.Stop();
                this.intervalTimer.Dispose();
                this.intervalTimer = null;
            }
        }

        /// <summary>
        /// オブジェクトを破棄する
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// オブジェクトを破棄する
        /// </summary>
        /// <param name="disposing">マネージドオブジェクトを破棄するか</param>
        protected virtual void Dispose(bool disposing)
        {
            if (this.disposed)
            {
                return;
            }

            if (disposing)
            {
                if (this.intervalTimer != null)
                {
                    this.intervalTimer.Stop();
                    this.intervalTimer.Dispose();
                }
            }

            this.disposed = true;
        }

        /// <summary>
        /// 情報を更新します。
        /// </summary>
        /// <remarks>DLLException以外の取得エラーではエラーメッセージ等は出さない</remarks>
        private void Update()
        {
            lock (this)
            {
                lock (this.ModelMain)
                {
                    this.NetInfo = null;
                    this.Nodes = null;
                    this.MyNodeInfo = null;
                    this.MySpeedInfo = null;

                    if (this.ModelMain.FLPCI == null)
                    {
                        return;
                    }

                    try
                    {
                        // ネットワークの状態、LKSの取得
                        this.NetInfo = this.ModelMain.FLPCI.GetNetInfo();
                        
                        // 参加ノード一覧取得
                        var nodes = this.ModelMain.FLPCI.GetRingInfo();
                        if (nodes != null)
                        {
                            // 自ノードを足し、ソートします
                            var nodes2 = new byte[nodes.Length + 1];
                            Array.Copy(nodes, nodes2, nodes.Length);
                            nodes2[nodes.Length] = this.ModelMain.NodeNumber;
                            Array.Sort(nodes2);
                            this.Nodes = nodes2;
                        }
                        
                        // 自ノードULS、自ノードの状態の取得
                        this.MyNodeInfo = this.ModelMain.FLPCI.GetNodeInfo(this.ModelMain.NodeNumber);

                        // 自ノード通信モードの取得
                        this.MySpeedInfo = this.ModelMain.FLPCI.GetSpeed();
                    }
                    catch (TimeoutException)
                    {
                        return;
                    }
                    catch (NotJoinException)
                    {
                        return;
                    }
                    catch (NodeNotJoinException)
                    {
                        return;
                    }
                    catch (CableException)
                    {
                        return;
                    }
                    catch (DLLException e)
                    {
                        this.ModelMain.ErrorMessageToViewModel = e.ToString();
                    }
                }
            }
        }

        /// <summary>
        /// タイマによって更新します。
        /// </summary>
        /// <param name="sender">イベント送信者</param>
        /// <param name="e">イベント引数</param>
        private void Update(object sender, ElapsedEventArgs e)
        {
            this.Update();
        }
    }
}
