﻿//-----------------------------------------------------------------------
// <copyright file="NodeInformation.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.ComponentModel;
using CenturySystems.FLPCI;
using CenturySystems.WPFSample.Common;

namespace CenturySystems.WPFSample.Model
{
    /// <summary>
    /// ノード情報のモデル
    /// </summary>
    public class NodeInformation : ModelBase
    {
        /// <summary>
        /// モデルメイン
        /// </summary>
        private ModelMain modelMain;

        /// <summary>
        /// ノード情報
        /// </summary>
        private NodeInfo nodeInfo;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="modelMain">モデルメイン</param>
        public NodeInformation(ModelMain modelMain)
        {
            this.ModelMain = modelMain;
        }

        /// <summary>
        /// モデルメイン
        /// </summary>
        public ModelMain ModelMain
        {
            get
            {
                return this.modelMain;
            }

            private set
            {
                this.modelMain = value;
                if (this.modelMain != null)
                {
                    // イベントを受ける設定
                    this.ModelPropertyChanged += this.OnListenerPropertyChanged;
                    PropertyChangedEventManager.AddListener(this.modelMain, this, string.Empty);
                }
            }
        }

        /// <summary>
        /// ノード情報
        /// </summary>
        public NodeInfo NodeInfo
        {
            get
            {
                return this.nodeInfo;
            }

            set
            {
                this.nodeInfo = value;
                this.RaisePropertyChanged("NodeInfo");
            }
        }

        /// <summary>
        /// ノード情報を更新できる状態かどうか(現状はFLPCIが初期化されていればtrue)
        /// </summary>
        public bool CanUpdateNodeInfo
        {
            get
            {
                if (this.modelMain == null)
                {
                    return false;
                }

                return this.modelMain.FLPCI != null;
            }
        }

        /// <summary>
        /// ノード情報を更新します。更新した結果はプロパティに反映され、イベントが発生します。
        /// </summary>
        /// <param name="nodeNumber">ノード番号</param>
        public void UpdateNodeInfo(byte nodeNumber)
        {
            if (this.modelMain == null)
            {
                throw new ApplicationException("modelMain is null");
            }

            if (this.modelMain.FLPCI == null) 
            {
                this.modelMain.ErrorMessageToViewModel = "FLPCI is not initialized";
                return;
            }

            try
            {
                this.NodeInfo = this.modelMain.FLPCI.GetNodeInfo(nodeNumber);
            }
            catch (ArgumentOutOfRangeException)
            {
                this.modelMain.ErrorMessageToViewModel = "Invalid node number.";
            }
            catch (NodeNotJoinException)
            {
                this.modelMain.ErrorMessageToViewModel = "Node is not join now.";
            }
            catch (TimeoutException)
            {
                this.modelMain.ErrorMessageToViewModel = "Timeout.";
            }
            catch (DLLException e)
            {
                this.modelMain.ErrorMessageToViewModel = e.ToString();
            }
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var modelMain = (ModelMain)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "CanCallFLPCIMethod":
                    // FLPCIが更新されたので、"CanUpdateNodeInfo"の状態も変わる
                    this.RaisePropertyChanged("CanUpdateNodeInfo");
                    break;
                default:
                    break;
            }
        }
    }
}
