﻿//-----------------------------------------------------------------------
// <copyright file="MainWindow.xaml.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.Windows;
using CenturySystems.WPFSample.ViewModel;

namespace CenturySystems.WPFSample.View
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window, IWeakEventListener
    {
        /// <summary>
        /// ビューモデルメイン
        /// </summary>
        private ViewModelMain viewModel;

        /// <summary>
        /// メインウィンドウ作成
        /// </summary>
        /// <param name="viewModelMain">ビューモデル</param>
        public MainWindow(ViewModelMain viewModelMain)
        {
            this.ViewModel = viewModelMain;

            this.InitializeComponent();

            // 各ビューのDataContextとしてビューモデルを設定する
            this.SetDataContextToChild("InitializeTab", viewModelMain.Initialize);
            this.SetDataContextToChild("CloseTab", viewModelMain.Close);
            this.SetDataContextToChild("AboutTab", viewModelMain.About);
            this.SetDataContextToChild("NodeInfoTab", viewModelMain.NodeInfo);
            this.SetDataContextToChild("SetULSTab", viewModelMain.SetULS);
            this.SetDataContextToChild("LogInfoTab", viewModelMain.LogInformation);
            this.SetDataContextToChild("MyCyclicTab", viewModelMain.MyCyclic);
            this.SetDataContextToChild("CommonMemoryTab", viewModelMain.CommonMemory);
            this.SetDataContextToChild("NetworkInformationTab", viewModelMain.NetworkInformation);
            this.SetDataContextToChild("MessageTab", viewModelMain.Message);
        }

        /// <summary>
        /// ビューモデルのプロパティが更新されたイベント
        /// </summary>
        private event PropertyChangedEventHandler ViewModelPropertyChanged;

        /// <summary>
        /// ビューモデル
        /// </summary>
        private ViewModelMain ViewModel
        {
            set
            {
                this.viewModel = value;
                if (this.viewModel != null)
                {
                    // イベントを受ける設定
                    this.ViewModelPropertyChanged += this.OnListenerPropertyChanged;
                    PropertyChangedEventManager.AddListener(this.viewModel, this, string.Empty);
                }
            }
        }

        /// <summary>
        /// ダイアログを開く
        /// </summary>
        /// <param name="message">メッセージ</param>
        public void OpenDialog(string message)
        {
            var dialog = new Dialog(message);
            try
            {
                dialog.Owner = this;
            }
            catch (InvalidOperationException)
            {
            }

            dialog.ShowDialog();
        }

        /// <summary>
        /// イベントを受信する
        /// </summary>
        /// <param name="managerType">マネージャタイプ</param>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        /// <returns>リスナーがイベントを処理したらtrue</returns>
        public bool ReceiveWeakEvent(Type managerType, object sender, EventArgs e)
        {
            if (managerType == typeof(PropertyChangedEventManager) && e is PropertyChangedEventArgs)
            {
                this.ViewModelPropertyChanged(sender, (PropertyChangedEventArgs)e);
            }
            else
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// 指定された名前の要素のデータコンテキストを設定します
        /// </summary>
        /// <param name="childName">名前</param>
        /// <param name="dataContext">データコンテキスト</param>
        private void SetDataContextToChild(string childName, object dataContext)
        {
            var tab = this.FindName(childName);
            if (tab == null)
            {
                throw new ApplicationException(childName + "is null");
            }

            if (tab is FrameworkElement)
            {
                var view = tab as FrameworkElement;
                view.DataContext = dataContext;
            }
            else
            {
                throw new ApplicationException(childName + " is not FrameworkElement");
            }
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var viewModelMain = (ViewModelMain)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "ErrorMessageToView":
                    this.OpenDialog(viewModelMain.ErrorMessageToView);
                    break;
                default:
                    break;
            }
        }
    }
}
