﻿//-----------------------------------------------------------------------
// <copyright file="Close.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System.ComponentModel;
using CenturySystems.WPFSample.Common;
using CenturySystems.WPFSample.Model;

namespace CenturySystems.WPFSample.ViewModel
{
    /// <summary>
    /// FL-PCI停止タブのビューモデル
    /// </summary>
    public class Close : ViewModelBase
    {
        /// <summary>
        /// モデル
        /// </summary>
        private ModelMain model;

        /// <summary>
        /// クローズするコマンド
        /// </summary>
        private DelegateCommand closeCommand;

        /// <summary>
        /// クローズするコマンド
        /// </summary>
        private DelegateCommand resetAll1Command;

        /// <summary>
        /// クローズするコマンド
        /// </summary>
        private DelegateCommand resetAll2Command;

        /// <summary>
        /// ボードのクローズが可能かどうかの状態
        /// </summary>
        private bool canClose;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public Close() : this(null)
        {
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="model">モデル</param>
        public Close(ModelMain model)
        {
            this.Model = model;   
        }

        /// <summary>
        /// モデル
        /// </summary>
        public ModelMain Model
        {
            get
            {
                return this.model;
            }

            set
            {
                this.model = value;
                if (this.model != null)
                {
                    // イベントで受ける変数の初期値を設定
                    this.canClose = this.model.CanCallFLPCIMethod;

                    // イベントの受け取りを設定
                    this.ModelPropertyChanged += this.OnListenerPropertyChanged;
                    PropertyChangedEventManager.AddListener(this.model, this, string.Empty);
                }
                else
                {
                    this.canClose = false;
                }
            }
        }

        /// <summary>
        /// ボードをクローズするコマンド
        /// </summary>
        public DelegateCommand CloseCommand
        {
            get
            {
                if (this.closeCommand == null)
                {
                    this.closeCommand = new DelegateCommand(this.CloseBoard, this.CanCloseBoard);
                }

                return this.closeCommand;
            }
        }

        /// <summary>
        /// ボード1の全てのデバイスをクローズしてボードをリセットするコマンド
        /// </summary>
        public DelegateCommand ResetAll1Command
        {
            get
            {
                if (this.resetAll1Command == null)
                {
                    this.resetAll1Command = new DelegateCommand(this.ResetAll1, this.CanResetAll);
                }

                return this.resetAll1Command;
            }
        }

        /// <summary>
        /// ボード2の全てのデバイスをクローズしてボードをリセットするコマンド
        /// </summary>
        public DelegateCommand ResetAll2Command
        {
            get
            {
                if (this.resetAll2Command == null)
                {
                    this.resetAll2Command = new DelegateCommand(this.ResetAll2, this.CanResetAll);
                }

                return this.resetAll2Command;
            }
        }

        /// <summary>
        /// ボードをクローズする
        /// </summary>
        public void CloseBoard()
        {
            this.model.CloseDevice(true);
        }

        /// <summary>
        /// ボードがクローズ可能かどうか
        /// </summary>
        /// <returns>ボードがクローズ可能ならtrue</returns>
        public bool CanCloseBoard()
        {
            return this.canClose;
        }

        /// <summary>
        /// ボード1の全てのデバイスをクローズしてボードをリセットする
        /// </summary>
        public void ResetAll1()
        {
            this.model.CloseAndResetAllDevice(1);
        }

        /// <summary>
        /// ボード2の全てのデバイスをクローズしてボードをリセットする
        /// </summary>
        public void ResetAll2()
        {
            this.model.CloseAndResetAllDevice(2);
        }

        /// <summary>
        /// 全てのデバイスをクローズしてボードをリセットすることが可能か
        /// </summary>
        /// <returns>常にtrue</returns>
        public bool CanResetAll()
        {
            return true;
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var modelMain = (ModelMain)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "CanCallFLPCIMethod":
                    this.canClose = modelMain.CanCallFLPCIMethod;
                    break;
                default:
                    break;
            }
        }
    }
}
