﻿//-----------------------------------------------------------------------
// <copyright file="CommonMemory.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using CenturySystems.FLNet;
using CenturySystems.WPFSample.Common;

namespace CenturySystems.WPFSample.ViewModel
{
    /// <summary>
    /// コモンメモリ用ビューモデル
    /// </summary>
    public class CommonMemory : ViewModelBase
    {
        /// <summary>
        /// モデル
        /// </summary>
        private Model.CommonMemory model;

        /// <summary>
        /// コモン1表示オフセット
        /// </summary>
        private ushort[] commonAddress = new ushort[] { 0, 0, 0 };

        /// <summary>
        /// コモン1表示数
        /// </summary>
        private ushort[] commonSize = FLNetUtil.CommonSizeMax;

        /// <summary>
        /// 実際のコモンメモリの内容
        /// </summary>
        private ushort[][] commonMemory = new ushort[3][];

        /// <summary>
        /// 自動更新を開始するコマンド
        /// </summary>
        private DelegateCommand startIntervalCommand;
        
        /// <summary>
        /// 自動更新を終了するコマンド
        /// </summary>
        private DelegateCommand stopIntervalCommand;

        /// <summary>
        /// 手動更新コマンド
        /// </summary>
        private DelegateCommand updateCommand;

        /// <summary>
        /// 自動更新を開始することが可能か
        /// </summary>
        private bool canStartInterval;

        /// <summary>
        /// 自動更新を終了することが可能か
        /// </summary>
        private bool canStopInterval;

        /// <summary>
        /// 手動更新することが可能か
        /// </summary>
        private bool canUpdate;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public CommonMemory()
            : this(null)
        {
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="model">モデル</param>
        public CommonMemory(Model.CommonMemory model)
        {
            this.Model = model;
        }

        /// <summary>
        /// モデル
        /// </summary>
        public Model.CommonMemory Model
        {
            get
            {
                return this.model;
            }

            private set
            {
                this.model = value;
                if (this.model != null)
                {
                    // イベントで受ける変数の初期値を設定
                    this.canStartInterval = this.model.CanStartInterval;
                    this.canStopInterval = this.model.CanStopInterval;
                    this.canUpdate = this.model.CanUpdate;

                    // イベントを受ける設定
                    this.ModelPropertyChanged += this.OnListenerPropertyChanged;
                    PropertyChangedEventManager.AddListener(this.model, this, string.Empty);
                }
                else
                {
                    this.canStartInterval = false;
                    this.canStopInterval = false;
                    this.canUpdate = false;
                }
            }
        }

        /// <summary>
        /// 表示するコモンデータのリスト
        /// </summary>
        public ObservableCollection<CommonData> CommonDataList1 { get; private set; }

        /// <summary>
        /// 表示するコモンデータのリスト
        /// </summary>
        public ObservableCollection<CommonData> CommonDataList2 { get; private set; }

        /// <summary>
        /// コモン領域1アドレス
        /// </summary>
        public string CommonAddress1
        {
            get
            {
                return this.commonAddress[1].ToString();
            }

            set
            {
                ushort number;
                ushort.TryParse(value, out number);
                if (number > FLNetUtil.CommonSizeMax[1] - 1)
                {
                    number = (ushort)(FLNetUtil.CommonSizeMax[1] - 1);
                }

                this.commonAddress[1] = number;
            }
        }

        /// <summary>
        /// コモン領域1サイズ
        /// </summary>
        public string CommonSize1
        {
            get
            {
                return this.commonSize[1].ToString();
            }

            set
            {
                ushort number;
                ushort.TryParse(value, out number);
                if (number > FLNetUtil.CommonSizeMax[1])
                {
                    number = (ushort)(FLNetUtil.CommonSizeMax[1]);
                }

                this.commonSize[1] = number;
            }
        }

        /// <summary>
        /// コモン領域2アドレス
        /// </summary>
        public string CommonAddress2
        {
            get
            {
                return this.commonAddress[2].ToString();
            }

            set
            {
                ushort number;
                ushort.TryParse(value, out number);
                if (number > FLNetUtil.CommonSizeMax[2] - 1)
                {
                    number = (ushort)(FLNetUtil.CommonSizeMax[2] - 1);
                }

                this.commonAddress[2] = number;
            }
        }

        /// <summary>
        /// コモン領域1サイズ
        /// </summary>
        public string CommonSize2
        {
            get
            {
                return this.commonSize[2].ToString();
            }

            set
            {
                ushort number;
                ushort.TryParse(value, out number);
                if (number > FLNetUtil.CommonSizeMax[2])
                {
                    number = (ushort)(FLNetUtil.CommonSizeMax[2]);
                }

                this.commonSize[2] = number;
            }
        }

        /// <summary>
        /// 自動更新を開始するコマンド
        /// </summary>
        public DelegateCommand StartIntervalCommand
        {
            get
            {
                if (this.startIntervalCommand == null)
                {
                    this.startIntervalCommand = new DelegateCommand(this.StartInterval, this.CanStartInterval);
                }

                return this.startIntervalCommand;
            }
        }

        /// <summary>
        /// 自動更新を停止するコマンド
        /// </summary>
        public DelegateCommand StopIntervalCommand
        {
            get
            {
                if (this.stopIntervalCommand == null)
                {
                    this.stopIntervalCommand = new DelegateCommand(this.StopInterval, this.CanStopInterval);
                }

                return this.stopIntervalCommand;
            }
        }

        /// <summary>
        /// 手動更新するコマンド
        /// </summary>
        public DelegateCommand UpdateCommand
        {
            get
            {
                if (this.updateCommand == null)
                {
                    this.updateCommand = new DelegateCommand(this.Update, this.CanUpdate);
                }

                return this.updateCommand;
            }
        }

        /// <summary>
        /// 自動更新を開始する(1秒)
        /// </summary>
        private void StartInterval()
        {
            this.MakeCommonDataList();
            this.Model.StartInterval(1000);
        }

        /// <summary>
        /// 自動更新の開始が可能か
        /// </summary>
        /// <returns>自動更新の開始が可能ならtrue</returns>
        private bool CanStartInterval()
        {
            return this.canStartInterval;
        }

        /// <summary>
        /// 自動更新を停止します。
        /// </summary>
        private void StopInterval()
        {
            this.Model.StopInterval();
        }

        /// <summary>
        /// 自動更新の停止が可能か
        /// </summary>
        /// <returns>自動更新の停止が可能ならtrue</returns>
        private bool CanStopInterval()
        {
            return this.canStopInterval;
        }

        /// <summary>
        /// 手動更新する
        /// </summary>
        private void Update()
        {
            this.MakeCommonDataList();
            this.Model.Update();
        }

        /// <summary>
        /// 手動更新が可能か
        /// </summary>
        /// <returns>手動更新が可能ならtrue</returns>
        private bool CanUpdate()
        {
            return this.canUpdate;
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var model = (Model.CommonMemory)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "CanStartInterval":
                    this.canStartInterval = model.CanStartInterval;
                    break;
                case "CanStopInterval":
                    this.canStopInterval = model.CanStopInterval;
                    break;
                case "CanUpdate":
                    this.canUpdate = model.CanUpdate;
                    break;
                case "Memory":
                    for (byte i = 1; i <= 2; i++)
                    {
                        this.commonMemory[i] = model.Memory[i];
                    }

                    this.ChangeCommonDataList();
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// CommonDataListの内容を作成します。
        /// </summary>
        private void MakeCommonDataList()
        {
            var cdl = new ObservableCollection<CommonData>[3] { null, this.CommonDataList1, this.CommonDataList2 };
            for (byte i = 1; i <= 2; i++)
            {
                if (cdl[i] == null)
                {
                    cdl[i] = new ObservableCollection<CommonData>();
                    switch (i)
                    {
                        case 1:
                            this.CommonDataList1 = cdl[i];
                            this.RaisePropertyChanged("CommonDataList1");
                            break;
                        case 2:
                            this.CommonDataList2 = cdl[i];
                            this.RaisePropertyChanged("CommonDataList2");
                            break;
                    }
                }
                else
                {
                    cdl[i].Clear();
                }

                for (int j = 0; j < this.commonSize[i]; j++)
                {
                    if (this.commonAddress[i] + j >= FLNetUtil.CommonSizeMax[i])
                    {
                        break;
                    }

                    cdl[i].Add(new CommonData((ushort)(this.commonAddress[i] + j), (ushort)(0)));
                }
            }
        }

        /// <summary>
        /// CommonDataListの内容を変更します。
        /// </summary>
        private void ChangeCommonDataList()
        {
            var cdl = new ObservableCollection<CommonData>[3] { null, this.CommonDataList1, this.CommonDataList2 };
            for (byte i = 1; i <= 2; i++)
            {
                for (int j = 0; j < this.commonSize[i]; j++)
                {
                    if (this.commonAddress[i] + j >= FLNetUtil.CommonSizeMax[i])
                    {
                        break;
                    }

                    cdl[i][j].Data = this.commonMemory[i][this.commonAddress[i] + j];
                }
            }
        }

        /// <summary>
        /// 個々のデータを表すクラス
        /// </summary>
        public class CommonData : INotifyPropertyChanged
        {
            /// <summary>
            /// データ(プロパティ変更イベントを起こすためにこの形にしている)
            /// </summary>
            private ushort data;

            /// <summary>
            /// コンストラクタ
            /// </summary>
            /// <param name="address">アドレス(絶対アドレス)</param>
            /// <param name="data">データ</param>
            public CommonData(ushort address, ushort data)
            {
                this.Address = address;
                this.Data = data;
            }

            /// <summary>
            /// プロパティが変わった場合のイベントハンドラ
            /// </summary>
            public event PropertyChangedEventHandler PropertyChanged;

            /// <summary>
            /// アドレス(絶対アドレス)
            /// </summary>
            public ushort Address { get; private set; }

            /// <summary>
            /// データ(プロパティ変更イベントを起こすためにこの形にしている)
            /// </summary>
            public ushort Data
            {
                get
                {
                    return this.data;
                }

                set
                {
                    this.data = value;
                    var h = this.PropertyChanged;
                    if (h != null)
                    {
                        h(this, new PropertyChangedEventArgs("Data"));
                    }
                }
            }
        }
    }
}
