﻿//-----------------------------------------------------------------------
// <copyright file="Initialize.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.Net;
using CenturySystems.FLPCI;
using CenturySystems.WPFSample.Common;
using CenturySystems.WPFSample.Model;

namespace CenturySystems.WPFSample.ViewModel
{
    /// <summary>
    /// 初期化のビューモデル
    /// </summary>
    public class Initialize : ViewModelBase
    {
        /// <summary>
        /// ビューモデルメイン
        /// </summary>
        private readonly ViewModelMain viewModelMain;

        /// <summary>
        /// モデル
        /// </summary>
        private ModelMain model;

        /// <summary>
        /// ボード番号(1-2)
        /// </summary>
        private byte boardNumber;

        /// <summary>
        /// 通信モード
        /// </summary>
        private Speed speed;

        /// <summary>
        /// IPアドレス
        /// </summary>
        private IPAddress ipAddress;

        /// <summary>
        /// コモン領域1アドレス
        /// </summary>
        private ushort common1Address;

        /// <summary>
        /// コモン領域1サイズ
        /// </summary>
        private ushort common1Size;

        /// <summary>
        /// コモン領域2アドレス
        /// </summary>
        private ushort common2Address;

        /// <summary>
        /// コモン領域2サイズ
        /// </summary>
        private ushort common2Size;

        /// <summary>
        /// トークン監視時間
        /// </summary>
        private byte tokenWatchTime;

        /// <summary>
        /// 最小許容フレーム間隔
        /// </summary>
        private byte minFrameTime;

        /// <summary>
        /// ノード名
        /// </summary>
        private string nodeName;

        /// <summary>
        /// ボードを初期化するコマンド
        /// </summary>
        private DelegateCommand initializeCommand;

        /// <summary>
        /// ボードの初期化が可能かどうかの状態
        /// </summary>
        private bool canInitialize;

        /// <summary>
        /// パラメータが全て正常にセットされているか
        /// </summary>
        private bool isParamSet;

        /// <summary>
        /// ビューモデルを初期化します。
        /// </summary>
        public Initialize()
            : this(null, null)
        {
        }

        /// <summary>
        /// ビューモデルを初期化します。
        /// </summary>
        /// <param name="model">モデル</param>
        /// <param name="viewModelMain">ビューモデルメイン</param>
        public Initialize(ModelMain model, ViewModelMain viewModelMain) 
        {
            this.boardNumber = 1;
            this.speed = Speed._10Mbps;
            this.IPAddressStr = "192.168.250.1";
            this.common1Address = 0;
            this.common1Size = 4;
            this.common2Address = 0;
            this.common2Size = 64;
            this.tokenWatchTime = 1;
            this.minFrameTime = 0;
            this.nodeName = string.Empty;
            this.Model = model;
            this.viewModelMain = viewModelMain;
            this.isParamSet = true;
        }

        /// <summary>
        /// モデル
        /// </summary>
        public ModelMain Model
        {
            get
            {
                return this.model;
            }

            set
            {
                this.model = value;
                if (this.model != null)
                {
                    // イベントで受ける変数の初期値を設定
                    this.canInitialize = this.model.CanInitializeBoard;

                    // イベントを受ける設定
                    this.ModelPropertyChanged += this.OnListenerPropertyChanged;
                    PropertyChangedEventManager.AddListener(this.model, this, string.Empty);
                }
                else
                {
                    this.canInitialize = false;
                }
            }
        }

        /// <summary>
        /// ボード1が選択された状態かどうか
        /// </summary>
        public bool Board1Selected
        {
            get
            {
                return this.boardNumber == 1;
            }

            set
            {
                if (value)
                {
                    this.boardNumber = 1;
                }
            }
        }

        /// <summary>
        /// ボード2が選択された状態かどうか
        /// </summary>
        public bool Board2Selected
        {
            get
            {
                return this.boardNumber == 2;
            }

            set
            {
                if (value)
                {
                    this.boardNumber = 2;
                }
            }
        }

        /// <summary>
        /// 10Mbpsが選択された状態かどうか
        /// </summary>
        public bool _10MbpsSelected
        {
            get
            {
                return this.speed == Speed._10Mbps;
            }

            set
            {
                if (value)
                {
                    this.speed = Speed._10Mbps;
                }
            }
        }

        /// <summary>
        /// 100Mbpsが選択された状態かどうか
        /// </summary>
        public bool _100MbpsSelected
        {
            get
            {
                return this.speed == Speed._100Mbps;
            }

            set
            {
                if (value)
                {
                    this.speed = Speed._100Mbps;
                }
            }
        }

        /// <summary>
        /// 自動選択が選択された状態かどうか
        /// </summary>
        public bool AutoNegotiationSelected
        {
            get
            {
                return this.speed == Speed.AutoNegotiation;
            }

            set
            {
                if (value)
                {
                    this.speed = Speed.AutoNegotiation;
                }
            }
        }

        /// <summary>
        /// IPアドレスの文字列
        /// </summary>
        public string IPAddressStr
        {
            get
            {
                return this.ipAddress.ToString();
            }

            set
            {
                try
                {
                    this.ipAddress = IPAddress.Parse(value);
                }
                catch (FormatException)
                {
                }
            }
        }

        /// <summary>
        /// コモン1アドレス
        /// </summary>
        public ushort Common1Address
        {
            get
            {
                return this.common1Address;
            }

            set
            {
                this.common1Address = value;
            }
        }

        /// <summary>
        /// コモン1サイズ
        /// </summary>
        public ushort Common1Size
        {
            get
            {
                return this.common1Size;
            }

            set
            {
                this.common1Size = value;
            }
        }

        /// <summary>
        /// コモン2アドレス
        /// </summary>
        public ushort Common2Address
        {
            get
            {
                return this.common2Address;
            }

            set
            {
                this.common2Address = value;
            }
        }

        /// <summary>
        /// コモン2サイズ
        /// </summary>
        public ushort Common2Size
        {
            get
            {
                return this.common2Size;
            }

            set
            {
                this.common2Size = value;
            }
        }

        /// <summary>
        /// トークン監視時間
        /// </summary>
        public byte TokenWatchTime
        {
            get
            {
                return this.tokenWatchTime;
            }

            set
            {
                this.tokenWatchTime = value;
            }
        }

        /// <summary>
        /// 最小許容フレーム間隔
        /// </summary>
        public byte MinFrameTime
        {
            get
            {
                return this.minFrameTime;
            }

            set
            {
                this.minFrameTime = value;
            }
        }

        /// <summary>
        /// ノード名
        /// </summary>
        public string NodeName
        {
            get
            {
                return this.nodeName;
            }

            set
            {
                this.nodeName = value;
            }
        }

        /// <summary>
        /// ボードを初期化するコマンド
        /// </summary>
        public DelegateCommand InitializeBoardCommand
        {
            get
            {
                if (this.initializeCommand == null)
                {
                    this.initializeCommand = new DelegateCommand(this.InitializeBoard, this.CanInitializeBoard);
                }

                return this.initializeCommand;
            }
        }

        /// <summary>
        /// ボードを初期化する
        /// </summary>
        public void InitializeBoard()
        {
            try
            {
                var nodeInitInfo = new NodeInitInfo(this.Common1Address, this.Common1Size, this.Common2Address, this.Common2Size, this.TokenWatchTime, this.MinFrameTime, this.NodeName);
                this.model.InitializeBoard(this.ipAddress, this.boardNumber, this.speed, nodeInitInfo);
            }
            catch (ArgumentOutOfRangeException e)
            {
                this.viewModelMain.ErrorMessageToView = "Invalid Parameter " + e.ParamName;
            }
        }

        /// <summary>
        /// ボードを初期化するコマンドが実行可能かどうか
        /// </summary>
        /// <returns>実行可能ならtrue</returns>
        public bool CanInitializeBoard()
        {
            return this.canInitialize && this.isParamSet;
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var modelMain = (ModelMain)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "CanInitializeBoard":
                    this.canInitialize = modelMain.CanInitializeBoard;
                    break;
                default:
                    break;
            }
        }
    }
}
