﻿//-----------------------------------------------------------------------
// <copyright file="Message.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Windows;
using System.Windows.Data;
using CenturySystems.FLNet;
using CenturySystems.FLPCI;
using CenturySystems.WPFSample.Common;

namespace CenturySystems.WPFSample.ViewModel
{
    /// <summary>
    /// メッセージのビューモデル
    /// </summary>
    public class Message : ViewModelBase, IWeakEventListener
    {
        /// <summary>
        /// モデル
        /// </summary>
        public readonly Model.Message Model;

        /// <summary>
        /// 送受信を開始するコマンド
        /// </summary>
        private DelegateCommand startCommCommand;

        /// <summary>
        /// 送受信を開始するコマンド
        /// </summary>
        private DelegateCommand stopCommCommand;

        /// <summary>
        /// 送信するコマンド
        /// </summary>
        private DelegateCommand sendMessageCommand;

        /// <summary>
        /// 送信データをリセットするコマンド
        /// </summary>
        private DelegateCommand resetMessageCommand;

        /// <summary>
        /// 送受信開始が可能か
        /// </summary>
        private bool canStartComm;

        /// <summary>
        /// 送受信停止が可能か
        /// </summary>
        private bool canStopComm;

        /// <summary>
        /// 送信が可能か
        /// </summary>
        private bool canSendMessage;

        /// <summary>
        /// VIEWに見せるメッセージリスト
        /// </summary>
        private ObservableCollection<MessageToView> messageToViewList;

        /// <summary>
        /// 送信するメッセージ
        /// </summary>
        private MessageInfo message;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public Message()
            : this(null)
        {
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="model">モデル</param>
        public Message(Model.Message model)
        {
            this.Model = model;
            if (this.Model != null)
            {
                // イベントで受ける変数の初期値を設定
                this.canStartComm = this.Model.CanStartComm;
                this.canStopComm = this.Model.CanStopComm;
                this.canSendMessage = this.Model.CanSendMessage;
                this.MessageToViewList = this.CreateMessageToViewList(this.Model.ReceivedMessage);

                // イベントを受ける設定
                this.ModelPropertyChanged += this.OnListenerPropertyChanged;
                PropertyChangedEventManager.AddListener(this.Model, this, string.Empty);

                // モデル内の受信メッセージのイベントを受ける
                this.MessageArrived += this.OnListenerMessageArrived;
                CollectionChangedEventManager.AddListener(this.Model.ReceivedMessage, this);

                this.MessageArrived2 += this.OnListenerMessageArrived2;
                PropertyChangedEventManager.AddListener(this.Model, this, string.Empty);
            }
            else
            {
                this.canStartComm = false;
                this.canStopComm = false;
                this.canSendMessage = false;
                this.MessageToViewList = new ObservableCollection<MessageToView>();
            }

            this.message = new MessageInfo(1, ResultCode.Normal, 65015, 0, 0, 0, new byte[0]);
        }

        /// <summary>
        /// モデルのプロパティの変更があったときに発行されます。
        /// </summary>
        public new event PropertyChangedEventHandler ModelPropertyChanged;

        /// <summary>
        /// receivedMessageが更新されたときのイベント
        /// </summary>
        public event NotifyCollectionChangedEventHandler MessageArrived;

        /// <summary>
        /// receivedMessageが更新されたときのイベント
        /// </summary>
        public event PropertyChangedEventHandler MessageArrived2;

        /// <summary>
        /// VIEWに見せるメッセージリスト
        /// </summary>
        public ObservableCollection<MessageToView> MessageToViewList
        {
            get
            {
                return this.messageToViewList;
            }

            private set
            {
                this.messageToViewList = value;
                BindingOperations.EnableCollectionSynchronization(this.messageToViewList, new object());
                this.RaisePropertyChanged("MessageToViewList");
            }
        }

        /// <summary>
        /// 受信を開始するコマンド
        /// </summary>
        public DelegateCommand StartCommCommand
        {
            get
            {
                if (this.startCommCommand == null)
                {
                    this.startCommCommand = new DelegateCommand(this.StartComm, this.CanStartComm);
                }

                return this.startCommCommand;
            }
        }

        /// <summary>
        /// 受信を停止するコマンド
        /// </summary>
        public DelegateCommand StopCommCommand
        {
            get
            {
                if (this.stopCommCommand == null)
                {
                    this.stopCommCommand = new DelegateCommand(this.StopComm, this.CanStopComm);
                }

                return this.stopCommCommand;
            }
        }

        /// <summary>
        /// 送信するコマンド
        /// </summary>
        public DelegateCommand SendMessageCommand
        {
            get
            {
                if (this.sendMessageCommand == null)
                {
                    this.sendMessageCommand = new DelegateCommand(this.SendMessage, this.CanSendMessage);
                }

                return this.sendMessageCommand;
            }
        }

        /// <summary>
        /// 送信データをリセットするコマンド
        /// </summary>
        public DelegateCommand ResetMessageCommand
        {
            get
            {
                if (this.resetMessageCommand == null)
                {
                    this.resetMessageCommand = new DelegateCommand(this.ResetMessage, this.CanResetMessage);
                }

                return this.resetMessageCommand;
            }
        }

        /// <summary>
        /// 送信メッセージノードNo.
        /// </summary>
        public string SendNode
        {
            get
            {
                return this.message.NodeNumber.ToString();
            }

            set
            {
                byte number;
                byte.TryParse(value, out number);
                try
                {
                    this.message.NodeNumber = number;
                }
                catch (ArgumentOutOfRangeException)
                {
                }
            }
        }

        /// <summary>
        /// 送信メッセージTCD
        /// </summary>
        public TCDCode SendTCD
        {
            get
            {
                var tcdCode = this.message.TCD;
                TCDCode codeObj;
                try
                {
                    codeObj = (TCDCode)Enum.ToObject(typeof(TCDCode), tcdCode);
                }
                catch (ArgumentException)
                {
                    codeObj = TCDCode.MessageLoopbackReq;
                }

                return codeObj;
            }

            set
            {
                this.message.TCD = (ushort)value;
            }
        }

        /// <summary>
        /// 送信メッセージデータ長
        /// </summary>
        public string SendBSize
        {
            get
            {
                return this.message.BSize.ToString();
            }

            set
            {
                ushort number;
                ushort.TryParse(value, out number);
                try
                {
                    this.message.BSize = number;
                }
                catch (ArgumentOutOfRangeException)
                {
                }
            }
        }

        /// <summary>
        /// 送信メッセージ仮想アドレス
        /// </summary>
        public string SendMAddr
        {
            get
            {
                return this.message.MAddr.ToString();
            }

            set
            {
                uint number;
                uint.TryParse(value, out number);
                this.message.MAddr = number;
            }
        }

        /// <summary>
        /// 送信メッセージ仮想サイズ
        /// </summary>
        public string SendMSize
        {
            get
            {
                return this.message.MSize.ToString();
            }

            set
            {
                ushort number;
                ushort.TryParse(value, out number);
                try
                {
                    this.message.MSize = number;
                }
                catch (ArgumentOutOfRangeException)
                {
                }
            }
        }

        /// <summary>
        /// モデルのプロパティ変更イベントを受け取ります。このクラスではCollectionChangedも受け取ります。
        /// </summary>
        /// <param name="managerType">マネージャタイプ</param>
        /// <param name="sender">イベント発行元</param>
        /// <param name="e">引数</param>
        /// <returns>イベントを処理したらtrue</returns>
        public new bool ReceiveWeakEvent(Type managerType, object sender, EventArgs e)
        {
            if (managerType == typeof(PropertyChangedEventManager) && sender is Model.Message && e is PropertyChangedEventArgs)
            {
                this.ModelPropertyChanged(sender, (PropertyChangedEventArgs)e);
            }
            else if (managerType == typeof(CollectionChangedEventManager) && e is NotifyCollectionChangedEventArgs)
            {
                this.MessageArrived(sender, (NotifyCollectionChangedEventArgs)e);
            }
            else if (managerType == typeof(PropertyChangedEventManager) && sender is Model.Message.MessageToViewModel && e is PropertyChangedEventArgs)
            {
                this.MessageArrived2(sender, (PropertyChangedEventArgs)e);
            }
            else 
            {
                Console.WriteLine("event: type " + managerType.ToString() + " sender " + sender.ToString() + " eventargs " + e.ToString());
                return false;
            }

            return true;
        }

        /// <summary>
        /// 受信を開始します
        /// </summary>
        private void StartComm()
        {
            this.Model.StartComm();
        }

        /// <summary>
        /// 受信の開始が可能か
        /// </summary>
        /// <returns>受信の開始が可能ならtrue</returns>
        private bool CanStartComm()
        {
            return this.canStartComm;
        }

        /// <summary>
        /// 受信を停止します
        /// </summary>
        private void StopComm()
        {
            this.Model.StopComm();
        }

        /// <summary>
        /// 受信の停止が可能か
        /// </summary>
        /// <returns>送受信の停止が可能ならtrue</returns>
        private bool CanStopComm()
        {
            return this.canStopComm;
        }

        /// <summary>
        /// 送信します
        /// </summary>
        private void SendMessage()
        {
            // サイズに従ってデータを生成する
            var length = Math.Max(this.message.MSize, this.message.BSize);
            var data = new byte[length];
            for (int i = 0; i < length; i++)
            {
                data[i] = (byte)(i & 0xff);
            }

            this.message.Data = data;
            this.Model.SendMessage(this.message);
        }

        /// <summary>
        /// 送信が可能か
        /// </summary>
        /// <returns>送信が可能ならtrue</returns>
        private bool CanSendMessage()
        {
            return this.canSendMessage;
        }

        /// <summary>
        /// 送信データをリセットします
        /// </summary>
        private void ResetMessage()
        {
            this.message = new MessageInfo(1, ResultCode.Normal, 65015, 0, 0, 0, new byte[0]);
            this.RaisePropertyChanged("SendNode");
            this.RaisePropertyChanged("SendTCD");
            this.RaisePropertyChanged("SendBSize");
            this.RaisePropertyChanged("SendMAddr");
            this.RaisePropertyChanged("SendMSize");
        }

        /// <summary>
        /// 送信データのリセットが可能か
        /// </summary>
        /// <returns>常にtruetrue</returns>
        private bool CanResetMessage()
        {
            return true;
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var model = (Model.Message)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "CanStartComm":
                    this.canStartComm = model.CanStartComm;
                    break;
                case "CanStopComm":
                    this.canStopComm = model.CanStopComm;
                    break;
                case "CanSendMessage":
                    this.canSendMessage = model.CanSendMessage;
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// モデルのメッセージ一覧からのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerMessageArrived(object sender, NotifyCollectionChangedEventArgs e)
        {
            var receivedMessageList = (ObservableCollection<Model.Message.MessageToViewModel>)sender;
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    foreach (var item in e.NewItems)
                    {
                        if (item is Model.Message.MessageToViewModel)
                        {
                            var element = (Model.Message.MessageToViewModel)item;
                            var id = (uint)(this.MessageToViewList.Count + 1);
                            var m = new MessageToView(id, element.TimeSpan, element.MessageInfo);
                            this.MessageToViewList.Add(m);
                        }
                    }
                    
                    break;
                default:
                    this.MessageToViewList = this.CreateMessageToViewList(receivedMessageList);
                    break;
            }
        }

        /// <summary>
        /// モデルのメッセージ一覧からのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerMessageArrived2(object sender, PropertyChangedEventArgs e)
        {
            var receivedMessageList = (ObservableCollection<Model.Message.MessageToViewModel>)sender;
            this.MessageToViewList = this.CreateMessageToViewList(receivedMessageList);
        }

        /// <summary>
        /// ビューに公開するメッセージリストを作成します。
        /// </summary>
        /// <param name="receivedMessage">モデルから受け取ったメッセージリスト</param>
        /// <returns>ビューに公開するメッセージリスト</returns>
        private ObservableCollection<MessageToView> CreateMessageToViewList(
            ObservableCollection<Model.Message.MessageToViewModel> receivedMessage)
        {
            var returnValue = new ObservableCollection<MessageToView>();
            uint id = 1;
            foreach (var item in receivedMessage)
            {
                var itemToView = new MessageToView(id++, item.TimeSpan, item.MessageInfo);
                returnValue.Add(itemToView);
            }

            return returnValue;
        }

        /// <summary>
        /// ビューに公開するメッセージ
        /// </summary>
        public class MessageToView
        {
            /// <summary>
            /// 元になるメッセージ
            /// </summary>
            public readonly MessageInfo Message;

            /// <summary>
            /// コンストラクタ
            /// </summary>
            /// <param name="id">1から始まる番号</param>
            /// <param name="timeSpan">送受信開始からの時間</param>
            /// <param name="messageInfo">メッセージ</param>
            public MessageToView(uint id, TimeSpan timeSpan, MessageInfo messageInfo)
            {
                this.ID = id;
                this.TimeSpan = timeSpan;
                this.Message = messageInfo;
            }

            /// <summary>
            /// メッセージ番号(1からの連番)
            /// </summary>
            public uint ID { get; private set; }

            /// <summary>
            /// 送受信スタートからの経過時間
            /// </summary>
            public TimeSpan TimeSpan { get; private set; }

            /// <summary>
            /// トランザクションコード
            /// </summary>
            public uint TCD
            {
                get
                {
                    return this.Message.TCD;
                }
            }

            /// <summary>
            /// 送信元ノードNo.
            /// </summary>
            public byte NodeNumber
            {
                get
                {
                    return this.Message.NodeNumber;
                }
            }

            /// <summary>
            /// 結果コード
            /// </summary>
            public ResultCode ResultCode
            {
                get
                {
                    return this.Message.ResultCode;
                }
            }

            /// <summary>
            /// 仮想アドレス
            /// </summary>
            public uint MAddr
            {
                get
                {
                    return this.Message.MAddr;
                }
            }

            /// <summary>
            /// 仮想サイズ
            /// </summary>
            public ushort MSize
            {
                get
                {
                    return this.Message.MSize;
                }
            }

            /// <summary>
            /// データサイズ
            /// </summary>
            public ushort BSize
            {
                get
                {
                    return this.Message.BSize;
                }
            }

            /// <summary>
            /// データ
            /// </summary>
            public string Data
            {
                get
                {
                    var dataString = new string[this.Message.Data.Length];
                    for (int i = 0; i < this.Message.Data.Length; i++)
                    {
                        dataString[i] = this.Message.Data[i].ToString("X2");
                    }

                    return string.Join(" ", dataString);
                }
            }
        }
    }

    /// <summary>
    /// TCDコードと文字列の関係
    /// </summary>
    public class TCDCodeString
    {
        /// <summary>
        /// 初期化子
        /// </summary>
        static TCDCodeString()
        {
            TCDCodeItems = new List<TCDCodeString>();
            TCDCodeString.AddItem(TCDCode.ByteBlockReadReq, "バイトブロックリード要求");
            TCDCodeString.AddItem(TCDCode.ByteBlockWriteReq, "バイトブロックライト要求");
            TCDCodeString.AddItem(TCDCode.WordBlockReadReq, "ワードブロックリード要求");
            TCDCodeString.AddItem(TCDCode.WordBlockWriteReq, "ワードブロックライト要求");
            TCDCodeString.AddItem(TCDCode.NetworkParameterReadReq, "ネットワークパラメタリード要求");
            TCDCodeString.AddItem(TCDCode.NetworkParameterWriteReq, "ネットワークパラメタライト要求");
            TCDCodeString.AddItem(TCDCode.StopReq, "停止指令要求");
            TCDCodeString.AddItem(TCDCode.RunReq, "運転指令要求");
            TCDCodeString.AddItem(TCDCode.LogDataReadReq, "ログリード要求");
            TCDCodeString.AddItem(TCDCode.LogDataClearReq, "ログクリア要求");
            TCDCodeString.AddItem(TCDCode.MessageLoopbackReq, "折り返し試験要求");
            TCDCodeString.AddItem(TCDCode.ProfileReadReq, "プロファイルリード要求");
            TCDCodeString.AddItem(TCDCode.TransparentMessage59999, "透過型メッセージ59999");
            TCDCodeItems.TrimExcess();
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="code">TCDコード</param>
        /// <param name="str">表示する文字列</param>
        private TCDCodeString(TCDCode code, string str)
        {
            this.Code = code;
            this.String = str;
        }

        /// <summary>
        /// 一連のアイテム
        /// </summary>
        public static List<TCDCodeString> TCDCodeItems { get; set; }

        /// <summary>
        /// TCDコード
        /// </summary>
        public TCDCode Code { get; set; }

        /// <summary>
        /// 表示する文字列
        /// </summary>
        public string String { get; set; }

        /// <summary>
        /// アイテムを追加する
        /// </summary>
        /// <param name="code">TCDコード</param>
        /// <param name="str">表示する文字列</param>
        private static void AddItem(TCDCode code, string str)
        {
            var item = new TCDCodeString(code, str);
            TCDCodeItems.Add(item);
        }
    }
}
