﻿//-----------------------------------------------------------------------
// <copyright file="NetworkInformation.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System.Collections.ObjectModel;
using System.ComponentModel;
using CenturySystems.FLPCI;
using CenturySystems.WPFSample.Common;

namespace CenturySystems.WPFSample.ViewModel
{
    /// <summary>
    /// ネットワーク情報のビューモデル
    /// </summary>
    public class NetworkInformation : ViewModelBase
    {
        /// <summary>
        /// モデル
        /// </summary>
        private Model.NetworkInformation model;

        /// <summary>
        /// 表示する参加ノード一覧
        /// </summary>
        private ObservableCollection<byte> nodes;

        /// <summary>
        /// ネットワークの状態
        /// </summary>
        private NetInfo netInfo;

        /// <summary>
        /// 自ノードの情報
        /// </summary>
        private NodeInfo myNodeInfo;

        /// <summary>
        /// 自ノードの速度情報
        /// </summary>
        private SpeedInfo mySpeedInfo;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public NetworkInformation()
            : this(null)
        {
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="model">モデル</param>
        public NetworkInformation(Model.NetworkInformation model)
        {
            this.Model = model;
        }

        /// <summary>
        /// モデル
        /// </summary>
        public Model.NetworkInformation Model
        {
            get
            {
                return this.model;
            }

            private set
            {
                this.model = value;
                if (this.model != null)
                {
                    // イベントを受ける設定
                    this.ModelPropertyChanged += this.OnListenerPropertyChanged;
                    PropertyChangedEventManager.AddListener(this.model, this, string.Empty);
                }
            }
        }

        /// <summary>
        /// 表示する参加ノード一覧
        /// </summary>
        public ObservableCollection<byte> Nodes
        {
            get
            {
                return this.nodes;
            }

            private set
            {
                this.nodes = value;
                this.RaisePropertyChanged("Nodes");
            }
        }

        /// <summary>
        /// ネットワークの状態
        /// </summary>
        public NetInfo NetInfo
        {
            get
            {
                return this.netInfo;
            }

            private set
            {
                this.netInfo = value;
                this.RaisePropertyChanged("NetInfo");
                this.RaisePropertyChanged("LKS");
            }
        }

        /// <summary>
        /// 自ノードの状態
        /// </summary>
        public NodeInfo MyNodeInfo
        {
            get
            {
                return this.myNodeInfo;
            }

            private set
            {
                this.myNodeInfo = value;
                this.RaisePropertyChanged("MyNodeInfo");
            }
        }

        /// <summary>
        /// 自ノードの通信モード
        /// </summary>
        public SpeedInfo MySpeedInfo
        {
            get
            {
                return this.mySpeedInfo;
            }

            private set
            {
                this.mySpeedInfo = value;
                this.RaisePropertyChanged("MySpeedInfo");
                this.RaisePropertyChanged("MySpeed10Mbps");
                this.RaisePropertyChanged("MySpeed100Mbps");
                this.RaisePropertyChanged("MySpeedFullDuplex");
                this.RaisePropertyChanged("MySpeedHalfDuplex");
            }
        }

        /// <summary>
        /// 自ノードの通信モードが10Mbpsか?
        /// </summary>
        public bool MySpeed10Mbps
        {
            get
            {
                if (this.MySpeedInfo != null)
                {
                    return this.MySpeedInfo.Speed == Speed._10Mbps;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// 自ノードの通信モードが100Mbpsか?
        /// </summary>
        public bool MySpeed100Mbps
        {
            get
            {
                if (this.MySpeedInfo != null)
                {
                    return this.MySpeedInfo.Speed == Speed._100Mbps;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// 自ノードの通信モードが全二重か?
        /// </summary>
        public bool MySpeedFullDuplex
        {
            get
            {
                if (this.MySpeedInfo != null)
                {
                    return this.MySpeedInfo.Duplex == Duplex.FullDuplex;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// 自ノードの通信モードが半二重か?
        /// </summary>
        public bool MySpeedHalfDuplex
        {
            get
            {
                if (this.MySpeedInfo != null)
                {
                    return this.MySpeedInfo.Duplex == Duplex.HalfDuplex;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// リンクの状態
        /// </summary>
        public FLPCILinkStatus LKS
        {
            get
            {
                if (this.netInfo != null)
                {
                    return this.netInfo.LinkStatus;
                }
                else
                {
                    return null;
                }
            }
        }

        /// <summary>
        /// 表示する参加ノード一覧を作成します。
        /// </summary>
        /// <param name="nodes">参加ノード一覧</param>
        /// <returns>参加ノード一覧の表示用データ</returns>
        private ObservableCollection<byte> MakeNodes(byte[] nodes)
        {
            var returnValue = new ObservableCollection<byte>();
            if (nodes == null)
            {
                return returnValue;
            }

            foreach (var item in nodes)
            {
                returnValue.Add(item);
            }

            return returnValue;
        }

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var model = (Model.NetworkInformation)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "Nodes":
                    this.Nodes = this.MakeNodes(model.Nodes);
                    break;
                case "NetInfo":
                    this.NetInfo = model.NetInfo;
                    break;
                case "MyNodeInfo":
                    this.MyNodeInfo = model.MyNodeInfo;
                    break;
                case "MySpeedInfo":
                    this.MySpeedInfo = model.MySpeedInfo;
                    break;
                default:
                    break;
            }
        }
    }
}
