﻿//-----------------------------------------------------------------------
// <copyright file="ViewModelMain.cs" company="Century Systems">
// Copyright (c) 2015 Century Systems Co., Ltd. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System.ComponentModel;
using CenturySystems.WPFSample;
using CenturySystems.WPFSample.Common;
using CenturySystems.WPFSample.Model;

namespace CenturySystems.WPFSample.ViewModel
{
    /// <summary>
    /// ビューモデル全てを所有するクラス。ModelMainからのエラーメッセージの変化イベントも受ける。
    /// </summary>
    public class ViewModelMain : ViewModelBase
    {
        /// <summary>
        /// モデル
        /// </summary>
        private ModelMain model;

        /// <summary>
        /// ビューに送るエラーメッセージ
        /// </summary>
        private string errorMessageToView;

        /// <summary>
        /// ビューモデル全てを構築して初期化する。
        /// </summary>
        /// <param name="model">モデル</param>
        public ViewModelMain(ModelMain model)
        {
            this.Model = model;

            // ビューモデル構築
            this.About = new About(model.Version);
            this.Initialize = new Initialize(model, this);
            this.Close = new Close(model);
            this.NodeInfo = new ViewModel.NodeInformation(model.NodeInformation);
            this.SetULS = new SetULS(model);
            this.LogInformation = new ViewModel.LogInformation(model.LogInformation);
            this.MyCyclic = new MyCyclic(model.MyCyclic);
            this.CommonMemory = new CommonMemory(model.CommonMemory);
            this.NetworkInformation = new NetworkInformation(model.NetworkInformation);
            this.Message = new Message(model.Message);
        }

        /// <summary>
        /// Aboutタブのビューモデル
        /// </summary>
        public About About { get; private set; }
        
        /// <summary>
        /// 初期化タブのビューモデル
        /// </summary>
        public Initialize Initialize　{ get; private set; }
       
        /// <summary>
        /// クローズタブのビューモデル
        /// </summary>
        public Close Close { get; private set; }
        
        /// <summary>
        /// ノード情報のビューモデル
        /// </summary>
        public NodeInformation NodeInfo { get; private set; }
        
        /// <summary>
        /// 自ノードULS設定のビューモデル
        /// </summary>
        public SetULS SetULS { get; private set; }

        /// <summary>
        /// 自ノードサイクリックデータ設定のビューモデル
        /// </summary>
        public MyCyclic MyCyclic { get; private set; }

        /// <summary>
        /// ログ情報のビューモデル
        /// </summary>
        public LogInformation LogInformation { get; private set; }

        /// <summary>
        /// コモンメモリのビューモデル
        /// </summary>
        public CommonMemory CommonMemory { get; private set; }

        /// <summary>
        /// ネットワーク情報のビューモデル
        /// </summary>
        public NetworkInformation NetworkInformation { get; private set; }

        /// <summary>
        /// メッセージのビューモデル
        /// </summary>
        public Message Message { get; private set; }

        /// <summary>
        /// ビューに送るエラーメッセージ
        /// </summary>
        public string ErrorMessageToView
        {
            get
            {
                return this.errorMessageToView;
            }

            set
            {
                this.errorMessageToView = value;
                this.RaisePropertyChanged("ErrorMessageToView");
            }
        }

        /// <summary>
        /// モデル
        /// </summary>
        private ModelMain Model
        {
            set
            {
                this.model = value;
                if (this.model != null)
                {
                    // イベントで受ける変数の初期値を設定
                    this.errorMessageToView = this.model.ErrorMessageToViewModel;

                    // イベントを受ける設定
                    this.ModelPropertyChanged += this.OnListenerPropertyChanged;
                    PropertyChangedEventManager.AddListener(this.model, this, string.Empty);
                }
                else
                {
                    this.ErrorMessageToView = string.Empty;
                }
            }
        }

        // --Method--------------------------------------------

        /// <summary>
        /// Modelからのプロパティ変更イベントを受け取ります。
        /// </summary>
        /// <param name="sender">イベント発生源</param>
        /// <param name="e">イベント引数</param>
        private void OnListenerPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var modelMain = (ModelMain)sender;
            var changedProperty = e.PropertyName;
            switch (changedProperty)
            {
                case "ErrorMessageToViewModel":
                    this.ErrorMessageToView = modelMain.ErrorMessageToViewModel;
                    break;
                default:
                    break;
            }
        }
    }
}
